        page
******************************************************
** Copyright 2006 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************

; -----------------------------------------------------------------------------
; File:         Barometer.asm
; Function:     Barometer Sensor Support (DS2438)
; Routines:     calbar: Discover/store barometer sensor ID.
;               barini: Initialize the barometer sensor for first time
;               iqbar:  Fire off A/D conversions for barometer
;               iqbar1: Convert sensor output to InHg and MlBar values
;               bab2am: Main line binary to ASCII conversion in InHg
;               bab2m:  Main line binary to ASCII conversion in MlBar
;               barrd0: Read barometer sensor page 0
;               barstt: Collect barometer stastics
;               newbar: Re-initialize barometer stats after 5-min roll
;               bardel: Calculates if pressure is rising, falling, or steady
;               avgbar: Compute averaged barometer
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      calbar
; Called By:    ssens
; Calls:        wrmess (write 2-line LCD message)
;               isrch (find 1-wire ID)
;               rdf64 (read flash 64-byte block)
;               wrf64 (write flash 64-byte block)
;               appera (erase flash block)
; Gazintas:     None
; Gazoutas:     Flash location fisbar is programmed
; Function:     This is the barometer sensor finder program.  This is a unique
;               sensor that either exists or not.  If it exists, program it
;               into flash, otherwise leave it unprogrammed
; -----------------------------------------------------------------------------

calbar: lda     #$26            ; battery monitor chip family ID
        jsr     i2srch          ; go find an IDs (maybe more than one)
        tsta                    ; make sure we got something
        beq     calbno          ; if nothing, ask to erase.
        cmp     #1              ; see if only one
        beq     calbak          ; if so, on to see what it was
        bra     calb2s          ; if two sensors, pick.

calbno: lda     #34             ; message #34 (No sensor found)
        jsr     wrmess          ; write it
calbq:  jsr     wfnkey          ; wait for next keypress
        cmp     #kup            ; Up means erase IDs
        beq     calber          ; if so, go erase them
        cmp     #kdown          ; Down means ignore
        beq     calbex          ; if so, leave w/o erasing
        bra     calbq           ; if neither, wait for valid key

calber: ldhx    #$ffff          ; erased value
        sthx    mndray          ; bytes 0,1
        sthx    mndray+2        ; bytes 2,3
        sthx    mndray+4        ; bytes 4,5
        sthx    mndray+6        ; bytes 6,7.  Write erased value below
        bra     calbak          ; go program it

;       Go here if there are two DS2428s (user needs to pick).  Since they
;       SHOULD have picked the humidity sensor first, I just need to pick the
;       ID that is not the humidity sensor.

calb2s: clrh                    ; 0...
        ldx     #8              ; wanna compare 8 bytes
calb2l: decx                    ; want 0 at end
        lda     mndray,x        ; get byte from just-read ID table
        cmp     fihum,x         ; compare to humidity ID table
        bne     calbak          ; if not same, we got a match
        decx                    ; otherwise decrement counter
        bne     calb2l          ; loop till all done

;       Got here because the value in mndray matches the humidity sensor,
;       therefore value in msg+20 is the one we want.

        ldhx    msg+20          ; get first two bytes
        sthx    mndray+0        ; transfer them
        ldhx    msg+22          ; get next two bytes
        sthx    mndray+2        ; transfer them
        ldhx    msg+24          ; get next two bytes
        sthx    mndray+4        ; transfer them
        ldhx    msg+26          ; get last two bytes
        sthx    mndray+6        ; transfer them

;       Whether the correct data was in mndray or transferred from msg+20
;       we get here with the correct ID in mndray.

;       We now know we have a sensor, go store information
;       To update the flash block that has other addresses we don't want to
;       mess with we copy the entire block to (msg), modify the barometer
;       address, then write the entire block back.
calbak: ldhx    #fpage0         ; the page to read
        jsr     rdf64           ; read it

calbaq: ldhx    mndray          ; get bytes 0,1
        sthx    msg+32          ; store them
        ldhx    mndray+2        ; get bytes 0,1
        sthx    msg+34          ; store them
        ldhx    mndray+4        ; get bytes 0,1
        sthx    msg+36          ; store them
        ldhx    mndray+6        ; get bytes 0,1
        sthx    msg+38          ; store them

        sei                     ; interrupts off
        ldhx    #fpage0         ; point to whole page again
        jsr     appera          ; erase it
        ldhx    #fpage0         ; point to whole page again
        jsr     wrf64           ; program altered values

        cli                     ; interrupts back on
calbex: rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      barini
; Called By:    rstrtn
; Calls:        barrd0 (read DS2438 page 0)
;               sndadr (send 1-wire address)
;               i_writ (send 1-wire data)
;               i_rdb (read 1-wire bit)
;               i_rst (1-wire bus reset)
; Gazintas:     None
; Gazoutas:     Initialized DS2438 for the barometer sensor
; Function:     This routine is in charge of initializing the DS2438 so we
;               can get barometer data as rapidly as possible.  This routine
;               first checks to see if the DS2438 is initialized and aborts
;               if it is.  Initialization consists of setting up the sensor
;               to convert the right voltage source as well as turn off all
;               of the alarm and current measuring capabilities.
; -----------------------------------------------------------------------------
barini: clra                    ; 0...
        sta     baval           ; clear the value
        sta     baval+1         ; clear
        sta     bavad           ; clear
        sta     bavad+1         ; clear
        sta     bacnt           ; clear
        sta     errbar          ; clear Bar. error counter
;       lda     #$0             ; set a to "0"
;       sta     bartbl          ; set first location of table to 0(msb)

; Clear out the Barometer table

        clra                    ; A=0
        clrh                    ; MSB=0
        ldx     #11             ; LSB=11 (12 bytes)
newblp: sta     bartbl,x        ; clear a byte
        decx                    ; decrement counter/index
        bpl     newblp          ; loop till all done.


;       clrh                    ; index MSB=0
;       ldx     #00             ; source offset
;newblp:lda     bartbl,x        ; get souce byte
;       sta     bartbl+1,x      ; write next byte
;       incx                    ; increment pointer
;       txa                     ; transfer x reg to accum
;       cmp     #11             ; see if at end
;       bne     newblp          ; do all old bytes

        lda     fibar           ; get barometer ID first byte
        cmp     #$26            ; see if family code exists
        bne     barex           ; if not $26, no sensor installed

        jsr     barrd0          ; read page 0 into mndray
        bne     barex           ; if read fails, abort
        lda     mndray          ; get byte 0, the config byte
        and     #$0f            ; look at 4 LSBs only
        beq     barex           ; if all 0's, part has been initialized

; We continue here if the DS2428 has not been initialized.  To initialize it
; we write scratchpad page 0 with all 0's, verify it, then copy it to the
; DS2438s internal EEPROM.

; First write all 0's to the scratchpad

        ldhx    #fibar          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$4e            ; write scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

        clrx                    ; clear our counter
barilp: pshx                    ; save x
        clra                    ; wanna write a 0
        jsr     i_writ          ; write it
        pulx                    ; restore X
        incx                    ; increment counter
        cpx     #8              ; see if done yet
        bne     barilp          ; do 'em all

; Now read the scratchpad back to verify it

        jsr     barrs0          ; re-read page 0 back
        lda     mndray          ; get byte 0, the config byte
        and     #$0f            ; look at 4 LSBs only
        bne     barini          ; if someting failed, try again

; Now that scratchpad has been written and verified, time to copy to EEPROM

        ldhx    #fibar          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$48            ; copy scratchpad to EEPROM command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

barplp: jsr     i_rdb           ; get a bit
        bcc     barplp          ; loop til we get a 1

        jsr     i_rst           ; do a bus reset

barex:  rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      iqbar
; Called By:    timirq
; Calls:        sndadr (send 1-wire address)
;               i_writ (write a 1-wire byte)
;               i_rst  (send 1-wire reset pulse)
;               waitms (wait milliseconds)
;               i_rdb  (read 1-wire bit)
; Gazintas:     None
; Gazoutas:     bavad (Barometer sensor output voltage)
; Function:     First check to see if we have a valid ID for the Barometer
;               sensor.  If not, no sensor is installed.  All this first
;               call does is fire off the conversions for the barometer
;               output voltage.  The latter is also
;               retrieved and restored before this code completes.
; -----------------------------------------------------------------------------
iqbar:  lda     fibar           ; get barometer ID first byte
        cmp     #$26            ; see if family code exists
        bne     iqbr1x          ; if not $26, no sensor installed

        lda     errbar          ; get error counter
        and     #$7f            ; get rid of error flag
        sta     errbar          ; save it in errbar

        ldhx    #fibar          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$4e            ; write scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it
        lda     #$00            ; Convert barometer sensor value
        jsr     i_writ          ; send it
        lda     #$00            ; Extra dummy value
        jsr     i_writ          ; send it

        ldhx    #fibar          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$44            ; convert tempature command
        jsr     i_writ          ; send command

bar1lp: lda     #1              ; 1 ms
        jsr     waitms          ; wait it
        jsr     i_rdb           ; get a bit
        bcc     bar1lp          ; loop til we get a 1

        ldhx    #fibar          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$b4            ; convert voltage command
        jsr     i_writ          ; send command

bar1l2: lda     #1              ; 1 ms
        jsr     waitms          ; wait it
        jsr     i_rdb           ; get a bit
        bcc     bar1l2          ; loop til we get a 1

        jsr     barrd0          ; go read results
        bne     iqba1r          ; if Z clear, error

        lda     mndray+2        ; get temperature MSB
        sta     bartmp          ; store it
        lda     mndray+1        ; get temperature LSB
        sta     bartmp+1        ; store it

; Now convert it to common t-238 temp format

        lda     bartmp+1         ; get temp LSB
        rola                     ; get 2^-1 in carry
        lda     bartmp           ; get temperature MSBs
        rola                     ; and make 7.1 format
        sta     bartmp          ; save it


        lda     mndray+4        ; get barometer sensor output MSB
        sta     bavad           ; store it
        lda     mndray+3        ; get barometer sensor output LSB
        sta     bavad+1         ; store it
        bra     iqbr1x          ; and exit

iqba1r: lda     errbar          ; get error counter
        inca                    ; increase counter
        ora     #$80            ; set 7 bit
        sta     errbar          ; store it

iqbr1x: rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      iqbar1
; Called By:    timirq
; Calls:        mul816
; Gazintas:     bavad
; Gazoutas:     bavaL (Barometer sensor output voltage*(0.429 for english, 0.144
;               for metric).
; Function:     This converts sensor voltage to pressure functions for metric
;               and english
; -----------------------------------------------------------------------------

iqbar1: lda     bavad           ; LOAD VALUE TO CONVERT
        sta     divdnd          ; by multiplying Bavd*0.429 (slope of line Bavad/Bp)
        lda     bavad+1
        sta     divdnd+1

        lda     #$6e            ; Load constant 0.429(6e binary)
        sta     divsor          ; store it for multiply
        jsr     mul816          ; go multiply (16 bits * 8 bits)
        lda     remndr          ; get results, MSB
        sta     baval           ; store in baval
        lda     quotnt          ; get LSB
        sta     baval+1         ; store iT

        tst     quotnt+1        ; see if we need to round up.
        bpl     iqbp11          ; if not, done.
        lda     baval+1         ; get LSB
        inca                    ; increment LSB
        sta     baval+1         ; get LSB
        bcc     iqbp11          ; if no carry, done
        lda     baval           ; get MSB
        inca                    ; increment LSB
        sta     baval           ; get MSB

iqbp11: lda     bavad           ; LOAD VALUE TO CONVERT
        sta     divdnd
        lda     bavad+1
        sta     divdnd+1

        lda     #$25            ; Load constant 0.144 (25 binary)
        sta     divsor          ; store it for multiply
        jsr     mul816          ; go multiply (16 bits * 8 bits)
        lda     remndr          ; get results, MSB(remaiander always 00 because of values used.)
        sta     bavalm          ; store in bavalm ( for metric value)
        lda     quotnt          ; get LSB
        sta     bavalm+1        ; store it

        tst     quotnt+1        ; see if we need to round up.
        bpl     iqbp12          ; if not, done.
        lda     bavalm+1        ; get LSB
        inca                    ; increment LSB
        sta     bavalm+1        ; get LSB
        bcc     iqbp12          ; if no carry, done
        lda     bavalm          ; get LSB
        inca                    ; increment LSB
        sta     bavalm          ; get LSB

iqbp12: rts                     ; RETURN


; -----------------------------------------------------------------------------
; Routine:      bab2am     THIS IS A MAINE LINE ROUTINE ONLY
; Called By:    multi
; Calls:        mul816 (8x16 multiply)
; Gazintas:     baval  (barometric pressure (Baval)
; Gazoutas:     matemp (0 to 3)  (barometric pressure in ASCII  xx.xx format)
;
; Function:     This routine takes the raw value gathered in the first
;               routine and computes the barometeric pressure in decimal format
;               by doing a series of countdown,
;               Baval= 27.46(Bavad*0.429)
;-----------------------------------------------------------------------------

bab2am: lda     baval+1         ; get convert value LSB
        add     #$ba            ; LSB of 2746
        sta     mbcnv+1         ; save it
        lda     baval           ; get convert value MSB
        adc     #$0a            ; MSB of 2746 (with carry)
        sta     mbcnv           ; save it

        mov     #'0',matemp+3   ; initialize thousands to ASCII Zero
bab2mi: ldhx    mbcnv           ; get conversion value
        cphx    #1000           ; see if thousands
        blo     bab2cc          ; done with thousands, on to hundreds
        inc     matemp+3        ; otherwise increment thousands

        lda     mbcnv+1         ; get LSB
        sub     #$e8            ; subtract 1000 LSB
        sta     mbcnv+1         ; save LSB
        lda     mbcnv           ; get MSB
        sbc     #$03            ; subtract 1000 MSB
        sta     mbcnv           ; save MSB result

        bra     bab2mi          ; and on to another thousands

bab2cc: sthx    mbcnv           ; save sub-1000 value to mbcnv so we can...
        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's
        jsr     bin2am          ; convert all of it to ASCII

; We now have the conversion value, just in the wrong digit order, it is in
; 2103 format instead of 3210 format.  Do some swizzling.

        lda     matemp+3        ; get (and save) thousands
        mov     matemp+2,matemp+3 ; move units
        mov     matemp+1,matemp+2 ; move tens
        mov     matemp+0,matemp+1 ; move hundreds
        sta     matemp+0        ; store thousands

        rts                     ; done

; -----------------------------------------------------------------------------
; Routine:      bab2m     THIS IS A MAINE LINE ROUTINE ONLY for METRIC
; Called By:    multi
; Calls:        mul816 (8x16 multiply)
; Gazintas:     bavad  (barometric pressure (Baval)
;
; Gazoutas:     matemp (0 to 3)  (barometric pressure in ASCII  xxxx format)
;
; Function:     This routine takes the raw value gathered in the first
;               routine and computes the barometeric pressure in METRIC
;               by doing a series of countdown,
;               Baval= 931+(Bavad*.1442)
;-----------------------------------------------------------------------------

bab2m:  lda     bavalm+1        ; get convert value LSB
        add     #$a3            ; LSB of 931
        sta     mbcnv+1         ; save result
        lda     bavalm          ; get convert value MSB
        adc     #$03            ; MSB of 931 (with carry)
        sta     mbcnv           ; save result

        mov     #'0',matemp+3   ; initialize thousands to ASCII Zero
bab2mm: ldhx    mbcnv           ; get value to convert
        cphx    #1000           ; see if thousands
        blo     bab2mc          ; done with thousands, on to hundreds
        inc     matemp+3        ; otherwise increment thousands

        lda     mbcnv+1         ; get LSB
        sub     #$e8            ; subtract 1000 LSB
        sta     mbcnv+1         ; save LSB
        lda     mbcnv           ; get MSB
        sbc     #$03            ; subtract 1000 MSB
        sta     mbcnv           ; save MSB result

        bra     bab2mm          ; and on to another thousands

bab2mc: sthx    mbcnv           ; save sub-1000 value to mbcnv so we can...
        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's
        jsr     bin2am          ; convert all of it to ASCII

; We now have the conversion value, just in the wrong digit order, it is in
; 2103 format instead of 3210 format.  Do some swizzling.

        lda     matemp+3        ; get (and save) thousands
        mov     matemp+2,matemp+3 ; move units
        mov     matemp+1,matemp+2 ; move tens
        mov     matemp+0,matemp+1 ; move hundreds
        sta     matemp+0        ; store thousands

        lda     #$30            ; need the extra zero
        sta     mtemp

        rts                     ; done

; -----------------------------------------------------------------------------
; Routine:      barrd0 - Read EEPROM page 0
; Routine:      barrs0 - Read Scratchpad Page 0
; Called By:    barini,iqbar0
; Calls:        sndadr (send 1-wire address)
;               i_writ (send 1-wire byte)
;               i_read (read 1-wire byte)
;               docrc8 (add byte to 8-bit CRC)
; Gazintas:     None
; Gazoutas:     mndray has 8 bytes from DS2438 page 0
;               Z set for successful read, Z clear for failure
; Function:     Read 8 bytes from DS2438 page 0
; -----------------------------------------------------------------------------
barrd0: ldhx    #fibar          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$b8            ; recall to scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

barrs0: ldhx    #fibar          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$be            ; read scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

        clr     crc8            ; clear 8-bit CRC

        jsr     i_read          ; Read byte 0
        sta     mndray+0        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 1
        sta     mndray+1        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 2
        sta     mndray+2        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 3
        sta     mndray+3        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 4
        sta     mndray+4        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 5
        sta     mndray+5        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 6
        sta     mndray+6        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 7
        sta     mndray+7        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; read CRC
        cmp     crc8            ; see if we match

        rts                     ; return with Z set if OK

; -----------------------------------------------------------------------------
; Routine       barstt - do Barometric Pressure gathering
; Called By:    timirq
; Calls:        None.
; Gazintas:     baval
; Gazoutas:     batot - cumulative sum of Bp values
;               bacnt - count of Bp values
; Function:     totalize up Bp values for averaging later.
;               Max binary value of pressure in Millbar 120, so 2 bytes are
;               plenty for 60 samples
; -----------------------------------------------------------------------------

barstt: lda     fibar           ; get Barometer ID first byte
        cmp     #$26            ; see if family code exists
        bne     bastx           ; if not $26, no sensor installed

        ; if error flag set, exit.
        lda     errbar          ; get error counter
        bmi     bastx           ; leave if error

        lda     bavalm+1        ; get barometric pressure
        add     batot+1         ; add cumulative LSB
        sta     batot+1         ; save LSB

        lda     batot           ; get MSB
        adc     bavalm          ; add MSBs
        sta     batot           ; save result

        lda     bacnt           ; get counter
        inca                    ; increment it
        sta     bacnt           ; and store it inc d8 doesn't work
bastx:  rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      newbar
; Called By:    timirq
; Calls:        None
; Gazintas:     None
; Gazoutas:     batot and baval initialized to 0
; Function:     Re-initialize Baro. pressure total values afer 5-minute roll
;               Roll the Bar table to keep track of pressure every 5 minutes
; -----------------------------------------------------------------------------
newbar: clra                    ; 0...
        sta     batot           ; totalized Barometer Sensor MSB
        sta     batot+1         ; totalized Barometer Sensor LSB
        sta     bacnt           ; sample counter

; The following 6 steps roll the batbl up one.
        clrh                    ; inxex MSB=0
        ldx     #10             ; source offset
nebrlp: lda     bartbl,x        ; get souce byte
        sta     bartbl+1,x      ; write one byte later
        decx                    ; decrement pointer
        bpl     nebrlp          ; do all old bytes

        lda     errbar
        cmp     #$80            ; see if most sig. bit set
        bhs     bartn           ; if error, done

;       Divide the 16-bit barometric pressure value by 4 for 8-bit value and
;       then stuff into the barometric pressure array.

        lda     bavad+1         ; get LSB of sensor output
        lsra                    ; shift lowest bit out.
        lsra                    ; shift next lowest bit out.
        sta     atemp           ; store it in atemp
        lda     bavad           ; get 2 lower bits or MSb OF SENSOR OUTPUT
        Ldx     #64             ; going to multiply by 64 to shift 2 lower bits to high
        mul                     ; multiply the too
        add     atemp           ; add lower bites

        sta     bartbl          ; save it

bartn:  rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      bardel
; Called By:    shoit
; Calls:        divu (integer unsigned division)
;               bin2as (convert binary to ASCII)
; Gazintas:     wx08b
; Gazoutas:     Direction message to disp 6
; Function:     Use Bartbl to determine if Barometer Pressure
;               has increased, decreased, or stayed constant over last hour
; -----------------------------------------------------------------------------
bardel: lda    bartbl+11       ; get last position in bartbl
        cmp    #$00            ; see if still zero
        beq    baenda          ; if not zero, work data

babeg1: lda    bartbl           ; get MSB
        cmp    bartbl+11        ; see what barometer read 1 hr ago
        beq    baenda           ; if equal, baro press is steady

BANDA:  sub    Bartbl+11        ; subtract( if barbl+23 larger than batbl+1, carry set)
        bcc     barise          ; if no carry bit, most recent baro. reading higher than 1hr. old

 ; Following is the routine that deals with a falling barometer

bafall: lda     #$06            ; the "falling" arrow
        jmp     baend           ; finished-return

; following is the routine that deals with a rising barometer

barise: lda     #$05           ; the "rising" arrow
        jmp     baend          ; finished-return

baenda: lda     #$7e           ; the "steady" arrow

baend:  rts

; -----------------------------------------------------------------------------
; Routine:      avgbar, tcba0a
; Called By:    tncmsg,cmpmsg2
; Calls:        divu (integer unsigned division)
;               bin2as (convert binary to ASCII)
; Gazintas:     batot (totalized barometric pressure)
;               hmcnt (barometer sample count)
; Gazoutas:     atemp (ASCII buffer with humidity value),temp
; Function:     Compute the averaged Bp for the past five minutes by
;               dividing the totalized barometric readings by the number of
;               readings in the past five minutes.
;
;               NOTE:  This is actually two routines.  The first (avgbar)
;               computes the average and converts it.  The second (tcba0a)
;               just does the conversion without averaging.
;
; -----------------------------------------------------------------------------
avgbar: clr     quotnt          ; default of 0 if errors
        clr     quotnt+1        ; default of 0 if errors

        lda     batot           ; get totalized Bp MSB
        sta     divdnd          ; save it for dividing
        lda     batot+1         ; get LSB
        sta     divdnd+1        ; save it too

        lda     bacnt           ; get sample count
        sta     divsor          ; to divide by
        beq     tcba0           ; if 0, skip division
        jsr     divu            ; totalized/sample=average
tcba0:  lda     quotnt          ; get  result MSB
        sta     divdnd          ; save result MSB
        lda     quotnt+1        ; get  result LSB
        sta     divdnd+1        ; save result LSB
        bra     tcba0b          ; go convert (skipping remainder clearing)

; *** ALTERNATE ENTRY POINT WITHOUT AVERAGING, INPUT IS IN DIVDND. ***

tcba0a: clr     remndr          ; make sure there is no remainder
tcba0b: lda     divdnd+1        ; get convert value LSB
        add     #$a3            ; LSB of 931
        sta     divdnd+1        ; save it
        lda     divdnd          ; get convert value MSB
        adc     #$03            ; MSB of 931 (with carry)
        sta     divdnd          ; save it.

        mov     #'0',atemp+3    ; initialize thousands to ASCII Zero
tcb2mm: ldhx    divdnd          ; get full value
        cphx    #1000           ; see if thousands
        blo     tcb2mc          ; done with thousands, on to hundreds
        inc     atemp+3         ; otherwise increment thousands

        lda     divdnd+1        ; get LSB
        sub     #$e8            ; subtract 1000 LSB
        sta     divdnd+1        ; save LSB
        lda     divdnd          ; get MSB
        sbc     #$03            ; subtract 1000 MSB
        sta     divdnd          ; save MSB result

        bra     tcb2mm          ; and on to another thousands

tcb2mc: sthx    divdnd          ; save sub-1000 value to mbcnv so we can...
        lda     #1              ; not 0...
        sta     zfill           ; want leading 0's
        jsr     bin2as          ; convert all of it to ASCII (remndr is preserved)

; We now have the conversion value, just in the wrong digit order, it is in
; 2103 format instead of 3210 format.  Do some swizzling.

        lda     atemp+3         ; get (and save) thousands
        mov     atemp+2,atemp+3 ; move units
        mov     atemp+1,atemp+2 ; move tens
        mov     atemp+0,atemp+1 ; move hundreds
        sta     atemp+0         ; store thousands

        lda     remndr          ; get remainder in 256ths of BP value division
        ldx     #10             ; scale to get 0-9
        mul                     ; multiply
        txa                     ; get 0-9 value in A
        add     #'0'            ; convert to ASCII
        sta     temp            ; save it

boeex:   rts
        
