        page
******************************************************
** Copyright 2006 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         humid.asm
; Function:     Humidity Sensor Support (DS2438)
; Routines:     calhum: Discover/store humidity sensor ID.
;               humini: Initialize the humidity sensor for first time
;               iqhum1: Fire off A/D conversions for humidity (1/2)
;               iqhum2: Fire off A/D conversions for humidity (2/2)
;               iqhum3: Compute relative humidity
;               humrd0: Read humidity sensor page 0
;               humnew: Re-initialize humidity stats after 5-min roll
;               avghum: Compute averaged humidity
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      calhum
; Called By:    ssens
; Calls:        wrmess (write 2-line LCD message)
;               isrch (find 1-wire ID)
;               rdf64 (read flash 64-byte block)
;               wrf64 (write flash 64-byte block)
;               appera (erase flash block)
; Gazintas:     None
; Gazoutas:     Flash location fishum is programmed
; Function:     This is the humidity sensor finder program.  This is a unique
;               sensor that either exists or not.  If it exists, program it
;               into flash, otherwise leave it unprogrammed
; -----------------------------------------------------------------------------

calhum: lda     #$26            ; battery monitor chip family ID
        jsr     i2srch          ; go find IDs (maybe more than one)
        tsta                    ; make sure we got something
        beq     calhno          ; if none, ask to erase ID
        cmp     #1              ; see if 1 sensor
        beq     calhmq          ; if so, no more decisions
        bne     calhm2          ; otherwise choices to make

calhno: lda     #34             ; message #34 (No sensor found)
        jsr     wrmess          ; write it
calhq:  jsr     wfnkey          ; wait for next keypress
        cmp     #kup            ; Up means erase IDs
        beq     calher          ; if so, go erase them
        cmp     #kdown          ; Down means ignore
        beq     calhex          ; if so, leave w/o erasing
        bra     calhq           ; if neither, wait for valid key

calher: ldhx    #$ffff          ; erased value
        sthx    mndray          ; bytes 0,1
        sthx    mndray+2        ; bytes 2,3
        sthx    mndray+4        ; bytes 4,5
        sthx    mndray+6        ; bytes 6,7.  Write erased value below
        stx     divdnd          ; clear temperature sensor source flag
        bra     calhmk          ; go program it

;       Two "26" sensors can exist, the humidity sensor as well as the BP
;       sensor.  Need to see if there are two "26" IDs.

calhm2: lda     #16             ; big message #16 (resolution choices)
        jsr     wr4mes          ; write it

calhml: jsr     wfnkey          ; wait for next keypress
        cmp     #kup            ; see if up key (remove a sensor)
        beq     calhum          ; if so, to see if one sensor now
        cmp     #kdown          ; see if down button (user pick)
        beq     calhmp          ; if so, check IDs again before cal
        bra     calhml          ; if neither, ignore.

;       We get here if the user chose to look at both IDs and decide which is
;       the real wind sensor.  This is now a shared routine.

calhmp: jsr     i2pick          ; let user pick, answer in (mndray)

;       now that we know there is a humidity sensor, time to ask the user
;       if they want to use its temperature sensor as the main sensor
;       or not.

calhmq: mov     #$ff,divsor     ; default to temperature from DS18x20
        lda     #11             ; big message #11 (Temp sensor choice)
        jsr     wr4mes          ; write it
calhq2: jsr     wfnkey          ; wait for next keypress
        cmp     #kup            ; Up means use humidity sensor
        beq     calhuh          ; if so, set flag
        cmp     #kdown          ; Down means use DS18x20
        beq     calhmk          ; if so, defalt OK, leave
        bra     calhq2          ; if neither, wait for valid key

calhuh: clr     divsor          ; set flag to use DS2450

;       To update the flash block that has other addresses we don't want to
;       mess with we copy the entire block to (msg), modify the wind speed
;       address, then write the entire block back.
calhmk: ldhx    #fpage0         ; the page to read
        jsr     rdf64           ; read it

        ldhx    mndray          ; get bytes 0,1
        sthx    msg+24          ; store them
        ldhx    mndray+2        ; get bytes 0,1
        sthx    msg+26          ; store them
        ldhx    mndray+4        ; get bytes 0,1
        sthx    msg+28          ; store them
        ldhx    mndray+6        ; get bytes 0,1
        sthx    msg+30          ; store them

        lda     divsor          ; get the Temp source Flag
        sta     msg+41          ; store it for Flash

        sei                     ; interrupts off
        ldhx    #fpage0         ; point to whole page again
        jsr     appera          ; erase it
        ldhx    #fpage0         ; point to whole page again
        jsr     wrf64           ; program altered values

        cli                     ; interrupts back on
calhex: rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      humini
; Called By:    rstrtn
; Calls:        humrd0 (read DS2438 page 0)
;               sndadr (send 1-wire address)
;               i_writ (send 1-wire data)
;               i_rdb (read 1-wire bit)
;               i_rst (1-wire bus reset)
; Gazintas:     None
; Gazoutas:     Initialized DS2438 for the humidity sensor
; Function:     This routine is in charge of initializing the DS2438 so we
;               can get humidity data as rapidly as possible.  This routine
;               first checks to see if the DS2438 is initialized and aborts
;               if it is.  Initialization consists of setting up the sensor
;               to convert the right voltage source as well as turn off all
;               of the alarm and current measuring capabilities.
; -----------------------------------------------------------------------------
humini: clr     errhum          ; clear humidity error counter
        clra                    ; 0...
        sta     hmval           ; clear the value
        sta     hmtot           ; totalized humidity MSB
        sta     hmtot+1         ; totalized humidity LSB
        sta     hmcnt           ; sample counter

        lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     humex           ; if not $26, no sensor installed

        jsr     humrd0          ; read page 0 into mndray
        bne     humex           ; if read fails, abort
        lda     mndray          ; get byte 0, the config byte
        and     #$0f            ; look at 4 LSBs only
        beq     humex           ; if all 0's, part has been initialized

; We continue here if the DS2428 has not been initialized.  To initialize it
; we write scratchpad page 0 with all 0's, verify it, then copy it to the
; DS2438s internal EEPROM.

; First write all 0's to the scratchpad

        ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$4e            ; write scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

        clrx                    ; clear our counter
humilp: pshx                    ; save x
        clra                    ; wanna write a 0
        jsr     i_writ          ; write it
        pulx                    ; restore X
        incx                    ; increment counter
        cpx     #8              ; see if done yet
        bne     humilp          ; do 'em all

; Now read the scratchpad back to verify it

        jsr     humrs0          ; re-read page 0 back
        lda     mndray          ; get byte 0, the config byte
        and     #$0f            ; look at 4 LSBs only
        bne     humini          ; if someting failed, try again

; Now that scratchpad has been written and verified, time to copy to EEPROM

        ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$48            ; copy scratchpad to EEPROM command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

humplp: jsr     i_rdb           ; get a bit
        bcc     humplp          ; loop til we get a 1

        jsr     i_rst           ; do a bus reset

humex:  rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      iqhum1
; Called By:    timirq
; Calls:        sndadr (send 1-wire address)
;               i_writ (write a 1-wire byte)
;               i_rst  (send 1-wire reset pulse)
;               waitms (wait milliseconds)
;               i_rdb  (read 1-wire bit)
; Gazintas:     None
; Gazoutas:     hmtmp (Humidity sensor temperature)
;               hmvad (Humidity sensor output voltage)
; Function:     First check to see if we have a valid ID for the humidty
;               sensor.  If not, no sensor is installed.  All this first
;               call does is fire off the conversions for the temperature
;               and humidity output voltage.  The latter two are also
;               retrieved and restored before this code completes.
; -----------------------------------------------------------------------------
iqhum1: lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     iqhm1x          ; if not $26, no sensor installed

        bclr    7,errhum        ; clear error flag

        ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$4e            ; write scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it
        lda     #$00            ; Convert Humidity sensor value
        jsr     i_writ          ; send it
        lda     #$00            ; Extra dummy value
        jsr     i_writ          ; send it

        ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$44            ; convert temperature command
        jsr     i_writ          ; send command

hum1lp: lda     #1              ; 1 ms
        jsr     waitms          ; wait it
        jsr     i_rdb           ; get a bit
        bcc     hum1lp          ; loop til we get a 1

        ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$b4            ; convert voltage command
        jsr     i_writ          ; send command

hum1l2: lda     #1              ; 1 ms
        jsr     waitms          ; wait it
        jsr     i_rdb           ; get a bit
        bcc     hum1l2          ; loop til we get a 1

        jsr     humrd0          ; go read results
        bne     iqhm1r          ; if Z clear, error

        lda     mndray+2        ; get temperature MSB
        sta     hmtmp           ; store it
        lda     mndray+1        ; get temperature LSB
        sta     hmtmp+1         ; store it

        lda     mndray+4        ; get humidity sensor output MSB
        sta     hmvad           ; store it
        lda     mndray+3        ; get humidity sensor output LSB
        sta     hmvad+1         ; store it
        bra     iqhm1x          ; exit with success

iqhm1r: inc     errhum          ; increment error counter
        bset    7,errhum        ; set error flag

iqhm1x: rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      iqhum2
; Called By:    timirq
; Calls:        sndadr (send 1-wire address)
;               i_writ (write a 1-wire byte)
;               i_rst  (send 1-wire reset pulse)
;               waitms (wait milliseconds)
;               i_rdb  (read 1-wire bit)
; Gazintas:     None
; Gazoutas:     hmvcc (Humidity sensor power supply voltage)
; Function:     First check to see if we have a valid ID for the humidty
;               sensor.  If not, no sensor is installed.  All this second
;               call does is fire off the conversions for the humidity
;               power supply voltage, wait for the convesion, and store the
;               result.
; -----------------------------------------------------------------------------
iqhum2: lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     iqhm2x          ; if not $26, no sensor installed

        brset   7,errhum,iqhm2x ; if error flag set, exit.

        ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$4e            ; write scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it
        lda     #$08            ; Convert VCC value
        jsr     i_writ          ; send it
        lda     #$00            ; Dummy value
        jsr     i_writ          ; send it

        ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$b4            ; convert voltage command
        jsr     i_writ          ; send command

hum2lp: lda     #1              ; 1 ms
        jsr     waitms          ; wait it
        jsr     i_rdb           ; get a bit
        bcc     hum2lp          ; loop til we get a 1

        jsr     humrd0          ; go read results
        bne     iqhm2r          ; if Z clear, error

        lda     mndray+4        ; get humidity sensor VCC MSB
        sta     hmvcc           ; store it
        lda     mndray+3        ; get humidity sensor VCC LSB
        sta     hmvcc+1         ; store it
        bra     iqhm2x          ; exit without error flags

iqhm2r: inc     errhum          ; increment error counter
        bset    7,errhum        ; set error flag

iqhm2x: rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      iqhum3
; Called By:    timirq
; Calls:        divf16 (16/16 fractional divide)
;               mul816 (8x16 multiply)
; Gazintas:     hmtmp  (Humidity temperature value)
;               hmvcc  (Humidity sensor power supply voltage)
;               hmvad  (Humidity sensor output voltage)
;               rdtemp (Current DS1820 temperature)
; Gazoutas:     ddtemp (User-preferred temperature (1820 or 2438)
;               hmval  (Humidity value)
;               hmdew  (dewpoint)
; Function:     This routine takes the raw values gathered in the first two
;               routines and computes the relative humidity using the following
;               formula:
;               hmval=(((hmvad/hmvcc)-0.16)/0.0062)/(1.0546-0.00216*hmtmp)
;               This seems simple enough to to where you have a C compiler
;               with floating point math, but alas I am not so lucky.
;
;               The sensor has some features that can result in innacurate
;               readings.  It is particularly succeptible to getting
;               "saturated" and giving readings greater than 100%.  This
;               routine limits results to a minimum of 1% on the low end and
;               100% on the high end.  Should the computed reading be less than
;               -4% or greater than 104%, I still limit the values, but set the
;               MSB of the humidity reading to indicate that we have exceeded
;               the published tolerances of the sensor.
;               For packet data, 0 is defined as 100%.  It is too often used to
;               represent "no reading", so I do not transmit that value.  A
;               value of 100 is changed to 99% in the UART routines.
;
;               In addition to calculating %RH, this routine also calculates
;               dewpoint by an equally nasty equation.  The comment block for
;               that is just before the math to figure it out
;
;               If the user has requested that the humidity sensors temperature
;               sensor be the official temperature sensor, the replacement
;               value gets written here.
; -----------------------------------------------------------------------------
;
;       For test purposes I have this code to preload "raw" values for the
;       sampled values in order to test out the code.  Here are the raw and
;       cooked values:
;       HMVAD HMVCC HMTMP hex/C/F %RH  Dewpoint C/F T238 %RH T238 DP C/F
;       ===== ===== ============= ==== ============ ======== =======
;       00d4  0192  e000 -32 -26   53  -39 / -38    53       -35
;       00d4  0192  2000 +32  90   60  +25 / +74    60       +27/80
;       00d4  0192  0000   0  32   56  -8  / +18    56       -2
;       0128  0174  0200   2  36   98  +2  / +35    --       --
; -----------------------------------------------------------------------------
$IF DEBUG
iqtest: lda     #$01            ; Preload VAD MSB
        sta     hmvad           ; store it
        lda     #$68            ; Preload VAD LSB
        sta     hmvad+1         ; store it

        lda     #$01            ; Preload VDD MSB
        sta     hmvcc           ; store it
        lda     #$a8            ; Preload VDD LSB
        sta     hmvcc+1         ; store it

        lda     #$fd            ; Preload temp MSB
        sta     hmtmp           ; store it
        lda     #$80            ; Preload temp LSB
        sta     hmtmp+1         ; store it

; Some other basic stuff to keep the simulator happy.
        clr     rdtemp          ; value unimportant, will use HMTMP
        clr     errtmp          ; value unimportant, will use ERRHUM
        clr     errhum          ; No errors for test purposes

$ENDIF
; -----------------------------------------------------------------------------
; Routine proper starts here...
; -----------------------------------------------------------------------------
iqhum3: clra                    ; 0...
        sta     hmval           ; clear the value (default)
        sta     hmdew           ; clear dewpoint too

;       We first need a common temperature as the user can select either the
;       DS18x20 (rdtemp) or DS2438 (hmtmp) as the official temperature
;       sensor.  I put the agreed temperaure in 7.1 (offset 110) format
;       into ddtemp.  This now becomes the official temperature.

        mov     rdtemp,ddtemp   ; make DS18x20 temperature default
        mov     errtmp,errdd    ; make DS18x20 error counter default too

        lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     iqh3lx          ; if not $26, no sensor installed, exit

        lda     fhmtmp          ; see which sensor to use
        bne     iqh3ce          ; if rdtemp, done, do error checking.

        lda     hmtmp+1         ; get temp LSB
        rola                    ; get 2^-1 in carry
        lda     hmtmp           ; get temperature MSBs
        rola                    ; and make 7.1 format
        add     #110            ; add offset
        sta     ddtemp          ; save it

        mov     errhum,errdd    ; just copy error counters

iqh3ce: brclr   7,errhum,iqh3go ; if no error, go compute humidity

iqh3lx: jmp     iqhm3x          ; if no sensor or error, exit

;       Enough pre-procesing, time to start computing humidity.
;       Divide Vad by Vcc first

iqh3go: lda     hmvad           ; Humidity sensor output MSB
        sta     divdnd          ; store it for dividing
        lda     hmvad+1         ; Humidity sensor output LSB
        sta     divdnd+1        ; store it for dividing

        lda     hmvcc           ; Humidity sensor power MSB
        sta     divsor          ; store it for dividing
        lda     hmvcc+1         ; Humidity sensor power LSB
        sta     remndr          ; store it for dividing

        jsr     divf16          ; divide the two, answer in quotnt

;       Next subtract 0.16 (147b in 1.15 format)

        lda     quotnt+1        ; get answer LSB
        sub     #$7b            ; subtract 0.16 LSB
        sta     quotnt+1        ; save result

        lda     quotnt          ; get answer MSB
        sbc     #$14            ; subtract 0.16 MSB
        sta     quotnt          ; save result

;       Put answer (quotnt) into divdnd so we can multiply
;       multiply answer above (in 1.15 format) by 161.29.
;       I'll use 8.0 format or $a1
;       Answer will be in 9.15 format

        lda     quotnt          ; Previous answer MSB
        sta     divdnd          ; move to argument
        lda     quotnt+1        ; Previous answer LSB
        sta     divdnd+1        ; move to argument
        lda     #$a1            ; multiplier
        sta     divsor          ; store it

        jsr     mul816          ; multiply

;       We now have the non-temperature-compensated humidity in
;       9.15 format.  Go ahead and save it in hmval for now while we
;       figger out the temperature compenstated value.

        lsl     quotnt          ; shift LSB of middle answer byte into C
        lda     remndr          ; get MSB answer
        rola                    ; shift into final answer
        tst     quotnt          ; check MSB of middle byte for rounding
        bpl     iqhm3a          ; if 0, don't increment
        inca                    ; otherwise round up.
iqhm3a: sta     hmval           ; save answer

;       Now go figger the temperature compensation value.  The rest of this
;       code will affect the reading by less than 5%.
;       First multiply the temperature (8.8 format) by 0.00216 ($008e in 0.16)
;       format.  We use an x256 value for .00216, so the 24-bit result will
;       be in 0.24 format.
;       One additional complication I forgot to take care of in the first
;       release. mul816 doesn't do negative numbers, so seperate this out and
;       do the multiplication with positive numbers and negate afterward if
;       needed.

        clr     temp            ; set +- flag to +
        lda     hmtmp           ; test temperature MSB
        bpl     iqhm3m          ; if positive, all is OK

        dec     temp            ; negative flag
        clra                    ; 0
        sub     hmtmp+1         ; subtract LSB
        sta     hmtmp+1         ; save negated LSB
        clra                    ; 0 again
        sbc     hmtmp           ; subtract MSB
        sta     hmtmp           ; save negated MSB

iqhm3m: lda     hmtmp           ; get temperature MSB
        sta     divdnd          ; put in multiplicand place
        lda     hmtmp+1         ; get temperature LSB
        sta     divdnd+1        ; put in multiplicand place
        lda     #$8e            ; LSB of .00126 in 0.16 format
        sta     divsor          ; save it

        jsr     mul816          ; multiplicate them.

        tst     temp            ; test the negative flag
        bpl     iqhm3s          ; if positive, all is OK

        clra                    ; 0
        sub     remndr          ; subtract MSB
        sta     remndr          ; save negated MSB
        bne     iqhm30          ; special-case a value of 0
        clr     temp            ; if 0, make MSB 0 too

; Negating hmtmp was pretty cheesy, so I need to re-negate it here to make it
; megative again so the correct value is presented to the rest of the routines
; that use this value.  I should be able to get away with this as you can't
; interrupt an interrupt routine.

iqhm30: clra                    ; 0
        sub     hmtmp+1         ; subtract LSB
        sta     hmtmp+1         ; save negated LSB
        clra                    ; 0 again
        sbc     hmtmp           ; subtract MSB
        sta     hmtmp           ; save negated MSB

;       With that number computed, take 1.0546 ($010e in 8.8 format) and
;       subtract the above computed value.  With an 8.8 format, all I
;       subtract is the MSB of the multiplied value.  The result gets stored
;       in divsor:remndr for the next operation.

iqhm3s: lda     #$0e            ; LSB of 1.0546
        sub     remndr          ; subtract MSB of temperature value
        sta     remndr          ; save result LSB
        lda     #$01            ; MSB of 1.0546
        sbc     temp            ; subtract MSB of temperature value
        sta     divsor          ; save result MSB

;       Now we have the value to divide by in 8.8 format.  Plunk in the
;       non-temperature compensated humitity also in 8.8 format and do the
;       division.

        lda     hmval           ; Humidity MSB
        sta     divdnd+1        ; save it
        clr     divdnd          ; LSB is 0

        jsr     divf16          ; do the division

;       Quotnt is the final temperature-compensated value in 8.8 format.
;       We only care about the MSB, so store it away in hmval.
;       Need to do the same x2 as before.

        lsl     quotnt+1        ; shift LSB of middle answer byte into C
        lda     quotnt          ; get MSB answer
        rola                    ; shift into final answer
        tst     quotnt+1        ; check MSB of middle byte for rounding
        bpl     iqhm3b          ; if 0, don't increment
        inca                    ; otherwise round up.
iqhm3b: sta     hmval           ; save answer

;       Now that we have the theoretical value, time to do some limits-ckecking
;       on it.  The sensor can (and does) give bad data, primarily when the
;       sensor saturates.  Large negative numbers are limited to 1%.
;       Values from 101-156 are assume to be a saturated sensor and just
;       limited to 100%.  See the header block for more details.

        lda     hmval           ; Yes, I know its redundant
        cmp     #251            ; This is essentially -4
        blo     iqhm31          ; if not, so far so good
        lda     #$01            ; otherwise limit to 1% with no error flag
        bra     iqhm39          ; all done

iqhm31: cmp     #200            ; Arbitrarily assume -56 is wrap-over to +200
        blo     iqhm32          ; if positive, then no error
        lda     #$81            ; neg value <-4% is limited to +1 with error flag
        bra     iqhm39          ; all done

iqhm32: cmp     #105            ; see if greater than 104%
        blo     iqhm33          ; if not, then no error
        lda     #228            ; >104% gets limited to 100% with error flag
        bra     iqhm39          ; all done

iqhm33: cmp     #101            ; see if greater than 100%
        blo     iqhm39          ; if not, then no error
        lda     #100            ; >100% & <105% limited to 99% without error flag

iqhm39: sta     hmval           ; save processed value

; -----------------------------------------------------------------------------
;       Now that we have the relative humidity figgered out, time to figger out
;       the dewpoint.  This is just about as messy as all the above routines.
;       According to Brent Shaw, the basic equation (in FORTRAN) is:
;         dewpt = t / (( -rvolv * ALOG(MAX(rh,0.01)) * t) + 1.0 )
;       Where:
;         rvolv = rv/lv (the constant below)
;         rv = 461.5   ! Gas constant for water vapor in J deg{-1} kg{-1}
;         lv = 2.500E6 ! Latent heat of vaporization in J kg{-1}
;         t  = Air temperature in deg K
;         rh = Relative humidity as a fraction (0.->1.)
;       Naturally this is all in floating point, so things get complicated
;       for us floating-point-challenged folk.
;
;       As I'm not into computing natural logs, I resort to a mix of fixed-
;       point arithmatic and some table-lookup.
;       Most of the nasty math surrounds the natural log routine.  The entire
;       "-rvolv * ALOG(MAX(rh,0.01))" part of the equation involves one input
;       variable (rh).  This can be replaced with a table that has RH as the
;       input (100 values) and a single 8-bit resultant value will yield
;       intermediate values that will have an overall <1% error from the
;       theoretical value.  This makes the mathematical part:
;       dewpt=t/((lookup(rh)*t)+1.0) where T is in Kelvin.
;
;       The real-world lookup vaules range from 0 to 0.000850, so a value of
;       255 corrosponds to 0.000850 and a value of 1 corrosponds to 0.0000033333.
;       Multiplied by 65536 gets us .21845 or .0011 0111 1110 1101 in 0.16
;       format.  I'm starting out with 8-bit values, so that is .0011 1000 00
;       in 0.10 format.  Multiplying by our lookup value (8.0) gets us an
;       answer in 6.10 format.
;
; -----------------------------------------------------------------------------
iqhmdp:
; For dewpoint I use the ddtemp temperature.  Alas when I did all this math
; I used hmtmp's format and really don't want to redo all the fixed-point
; math, so here I convert ddtemp to hmtmp format and put it back in hmtmp.

        lda     ddtemp          ; get temperature (7.1 format)
        sub     #110            ; get rid of offset
        asra                    ; Have MSByte in 8.0 format
        sta     hmtmp           ; save MSByte
        clra                    ; Start LSByte as 0's
        rora                    ; put the .1 bit in the LSByte MSBit
        sta     hmtmp+1         ; save LSByte

; The nasty math begins again...

        lda     hmval           ; get humidity value (1-99)
        and     #$7f            ; strip off any error flag
        sub     #1              ; table index 0 is for 1% humidity
        tax                     ; make an index

        clrh                    ; make sure index MSB is 0
        lda     dewtab,x        ; get our intermediate value (0-255)
        ldx     #$e0            ; muliplier (explained above)
        mul                     ; multiplicate it (answer is in 10.6 (*2^16))
        stx     divdnd          ; save MSB for multiply soon
        sta     divdnd+1        ; save LSB for multiply soon

;       Now multiply the lookup value by the current temperature
;       Lookup value is in 6.10 (*2^16)
;       Temperature will be in 8.0 (*2) format (and convert C to K)
;       After multiply, answer will be in 14.10 format (*2^17)
;       --or now we can say 0.26 format.

        lda     hmtmp           ; get the current temperature
        asra                    ; divide by two
        add     #$89            ; add 273/2 (now guaranteed to be positive!)
        sta     divsor          ; save as 8-bit multiplier
        jsr     mul816          ; multiplicate it

;       Largest practical value for the above intermediate result is
;       about 0.26, so adding 1 is pretty significant.  Add it.  Truncate to
;       16-bit math for now.  I need to shift right three bits to get 1.23
;       format first.

        lda     remndr          ; get MSB
        ldx     quotnt          ; get LSB
        asra                    ; shift right MSB 1x
        rorx                    ; shift into LSB
        asra                    ; shift right MSB 2x
        rorx                    ; shift into LSB
        ora     #$80            ; add the 1

        sta     remndr          ; save MSB
        stx     quotnt          ; save LSB

;       We now have the number to divide the temperature by to get the
;       relative humidity.  It is in 1.23 format.  Get the temperature again,
;       convert to Kelvin and divide by our derived number to get the
;       dewpoint in Kelvin.

;       We first get the temperature, divide it by two, then add 273 to it
;       to get degrees K in a 16-bit format.  The final format is 9.7
;       Next we take our derived number above in 1.23 format.
;       My division routine can only take an 8-bit divisor, so the MSB
;       becomes 1.7 format.
;       If I did things right, the answer will be in 9.7 format.

        ldx     hmtmp           ; temperature MSB
        lda     hmtmp+1         ; temperature LSB
        asrx                    ; divide MSB by two
        rora                    ; divide LSB too
        add     #$80            ; LSB of 273/2
        sta     divdnd+1        ; save it
        txa                     ; get MSB
        adc     #$88            ; MSB of 273/2
        sta     divdnd          ; Save MSB

        lda     remndr          ; get divisor MSB
        sta     divsor          ; save it
        lda     quotnt          ; get divisor LSB
        sta     remndr          ; save it
        jsr     divf16          ; do the division

;       After all that the dewpoint in degrees K is in quotent in 9.7 format.
;       Subtract 273 and shift to get it into signed S7.0 format.

        lda     quotnt+1        ; get degrees K/2 LSB
        sub     #$80            ; subtract 273/2 LSB
        sta     quotnt+1        ; save result

        lda     quotnt          ; get degrees K/2 MSB
        sbc     #$88            ; subtract 273/2 MSB
        sta     quotnt          ; save result

;       now convert from temperature /2 to temperature *2 and add offset-110

        lda     quotnt          ; get MSB again (x2 value)
        ldx     quotnt+1        ; get LSB again

        lslx                    ; LSB x2
        rola                    ; put in MSB
        lslx                    ; LSB x4
        rola                    ; put in MSB

        add     #110            ; convert to offset 110
        sta     hmdew           ; save the answer

iqhm3x: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine      humstt - do humidity statistics gathering
; Called By:    timirq
; Calls:        None.
; Gazintas:     hmval
; Gazoutas:     hmtot - cumulative sum of humidity values
;               hmcnt - count of humdity values
; Function:     totalize up humidity values for averaging later.
;               max hmcnt is 60, so 1 byte is fine
;               max hmtot is 60*100 or 6000, so 2 bytes is fine.
;
;               hmval is already limited to 1-100.  This just adds them up
;               and division is done later.  I still add up values that are
;               out of range so that even if I get 60 samples of >104% I still
;               want to report saturated humidity.
; -----------------------------------------------------------------------------

humstt: lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     hmstx           ; if not $26, no sensor installed

        brset   7,errhum,hmstx  ; if error flag set, exit.

        lda     hmval           ; get humidty
        and     #$7f            ; peel off "over range" flag
        add     hmtot+1         ; add cumulative LSB
        sta     hmtot+1         ; save LSB
        lda     hmtot           ; get MSB
        adc     #0              ; add carry (if any)
        sta     hmtot           ; save result

        lda     hmcnt           ; get counter
        add     #1              ; increment it
        sta     hmcnt           ; save result (can't INC byte address >$100)

hmstx:  rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      humrd0 - Read EEPROM page 0
; Routine:      humrs0 - Read Scratchpad Page 0
; Called By:    humini,iqhum0,iqhum2
; Calls:        sndadr (send 1-wire address)
;               i_writ (send 1-wire byte)
;               i_read (read 1-wire byte)
;               docrc8 (add byte to 8-bit CRC)
; Gazintas:     None
; Gazoutas:     mndray has 9 bytes from DS2438 page 0
;               Z set for successful read, Z clear for failure
; Function:     Read 8 bytes from DS2438 page 0
; -----------------------------------------------------------------------------
humrd0: ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$b8            ; recall to scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

humrs0: ldhx    #fihum          ; address for DS2438
        jsr     sndadr          ; send it off
        lda     #$be            ; read scratchpad command
        jsr     i_writ          ; send command
        lda     #$00            ; page 0
        jsr     i_writ          ; send it

        clr     crc8            ; clear 8-bit CRC

        jsr     i_read          ; Read byte 0
        sta     mndray+0        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 1
        sta     mndray+1        ; store it
        jsr     docrc8          ; do 8-bit CRC

        jsr     i_read          ; Read byte 2
        sta     mndray+2        ; store it
        jsr     docrc8          ; do 8-bit CRC 

        jsr     i_read          ; Read byte 3
        sta     mndray+3        ; store it
        jsr     docrc8          ; do 8-bit CRC 

        jsr     i_read          ; Read byte 4
        sta     mndray+4        ; store it
        jsr     docrc8          ; do 8-bit CRC 

        jsr     i_read          ; Read byte 5
        sta     mndray+5        ; store it
        jsr     docrc8          ; do 8-bit CRC 

        jsr     i_read          ; Read byte 6
        sta     mndray+6        ; store it
        jsr     docrc8          ; do 8-bit CRC 

        jsr     i_read          ; Read byte 7
        sta     mndray+7        ; store it
        jsr     docrc8          ; do 8-bit CRC 

        jsr     i_read          ; read CRC
        cmp     crc8            ; see if we match

        rts                     ; return with Z set if OK

        page
; -----------------------------------------------------------------------------
; Routine:      newhum
; Called By:    timirq
; Calls:        None
; Gazintas:     None
; Gazoutas:     hmtot and hmval initialized to 0
; Function:     Re-initialize humidity values afer 5-minute roll
; -----------------------------------------------------------------------------
newhum: clra                    ; 0...
        sta     hmval           ; clear the value
        sta     hmtot           ; totalized humidity MSB
        sta     hmtot+1         ; totalized humidity LSB
        sta     hmcnt           ; sample counter
        rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      avghum
; Called By:    tncmsg
; Calls:        divu (integer unsigned division)
;               bin2as (convert binary to ASCII)
; Gazintas:     hmtot (totalized humidity)
;               hmcnt (humidity sample count)
; Gazoutas:     atemp (ASCII buffer with humidity value)
; Function:     Compute the averaged humidity for the past five minutes by
;               dividing the totalized humidity readings by the number of
;               readings in the past five minutes.
;               If the value is exactly 100, then we move it down to 99
;               so that we do not report a value of 100 (used too often for
;               meaning "no reading"
; -----------------------------------------------------------------------------
avghum: clr     quotnt+1        ; default of 0 if errors
        lda     hmtot           ; get totalized humidity MSB
        sta     divdnd          ; save it for dividing
        lda     hmtot+1         ; get LSB
        sta     divdnd+1        ; save it too
        lda     hmcnt           ; get sample count
        sta     divsor          ; to divide by
        beq     tnchm0          ; if 0, skip division
        jsr     divu            ; totalized/sample=average
tnchm0: lda     quotnt+1        ; get result (0-100)
        cmp     #100            ; see if exaclty 100%
        bne     tnchm1          ; if not, all done
        lda     #99             ; change 100% to 99%
tnchm1: sta     divdnd+1        ; place to convert
        clr     divdnd          ; make sure MSB is 0
        jsr     bin2as          ; convert to ASCII

        rts                     ; all done

