        page
******************************************************
** Copyright 2006 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         lcd4.asm
; Function:     LCD drivers using 4-bit interface
; Routines:     wrmess: Write full 2x20 screen to the LCD
;               wr4mes: Write full 4x20 screen to the LCD
;               wcgram: Write CGRAM character bitmap
;               wrline: Write a single line of data to LCD.
;               wrstr:  Write a null-terminated string to LCD.
;               wrdat:  Write a byte of data to the LCD.
;               wrdats: Write a sequence of bytes to the LCD data port.
;               wrmstr: Write a (potentially) mulit-line null-terminated string.
;               wrcmd:  Write a command byte to the LCD.
;               wrini:  Write an initialization byte to the LCD.
;               ckrdy:  Check busy-bit on LCD controller
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      wrmess
; Called By:    lotsa places
; Calls:        wrcmd (write a command byte to the LCD)
;               wrline (write a line of data to the LCD)
; Gazintas:     A = message number
; Gazoutas:     None
; Function:     This routine will write a complete message to the LCD display.
;               The message number is passed in in the A register.  From this
;               number, the address of the message is calculated and its two
;               lines sent to the display.
;               The message array should consist of a bunch of 40-byte entries
;               with the first one referenced by an index of 0 and so on.
;               The table lable is banner
; -----------------------------------------------------------------------------
wrmess: ldx     #40             ; 40 characters per message
        mul                     ; x:a is now offset to message
        add     #banner%256     ; get message #0 address LSB
        sta     addrlo          ; save LSB

        txa                     ; get offset MSB
        adc     #banner\256     ; get message #0 MSB
        sta     addrhi          ; save MSB

        lda     #1              ; clear display message
        jsr     wrcmd           ; do it.

        lda     #$80            ; line 1 column 1...
        jsr     wrcmd           ; set it
        bsr     wrline          ; write line # 1

        lda     #$c0            ; line 2 column 1...
        jsr     wrcmd           ; set it
        bsr     wrline          ; write line # 2

        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      wr4mes
; Called By:    lotsa places
; Calls:        wrcmd (write a command byte to the LCD)
;               wrline (write a line of data to the LCD)
; Gazintas:     A = message number
; Gazoutas:     None
; Function:     This routine will write a complete message to the LCD display.
;               The message number is passed in in the A register.  From this
;               number, the address of the message is calculated and its two
;               lines sent to the display.
;               The message array should consist of a bunch of 80-byte entries
;               with the first one referenced by an index of 0 and so on.
;               The table lable is bbnnrs
; -----------------------------------------------------------------------------
wr4mes: ldx     #80             ; 40 characters per message
        mul                     ; x:a is now offset to message
        add     #bbnnrs%256     ; get message #0 address LSB
        sta     addrlo          ; save LSB

        txa                     ; get offset MSB
        adc     #bbnnrs\256     ; get message #0 MSB
        sta     addrhi          ; save MSB

        lda     #1              ; clear display message
        jsr     wrcmd           ; do it.

        lda     #$80            ; line 1 column 1...
        jsr     wrcmd           ; set it
        bsr     wrline          ; write line # 1

        lda     #$c0            ; line 2 column 1...
        jsr     wrcmd           ; set it
        bsr     wrline          ; write line # 2

        lda     #$94            ; line 3 column 1...
        jsr     wrcmd           ; set it
        bsr     wrline          ; write line # 1

        lda     #$d4            ; line 4 column 1...
        jsr     wrcmd           ; set it
        bsr     wrline          ; write line # 2

        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      wcgram
; Called By:    rstrtn
; Calls:        wrcmd (write a command byte to the LCD)
;               wrdat (write a data byte to the LCD)
; Gazintas:     A = index number for CGRAM entry
; Gazoutas:     None
; Function:     This routine writes a character to CGRAM.  The format in the
;               table is the command byte followed by eight data bytes.
; -----------------------------------------------------------------------------
wcgram: ldx     #9              ; 9 characters per message
        mul                     ; x:a is now offset to message

        add     #cgram0%256     ; add message #0 LSB
        sta     addrlo          ; and save it

        txa                     ; MSBs next
        adc     #cgram0\256     ; get message #0 MSB
        sta     addrhi          ; addr__ now contains address of message

        ldhx    addrhi          ; get data address
        lda     ,x              ; get the data to send
        aix     #1              ; increment address
        sthx    addrhi          ; save addr for later
        jsr     wrcmd           ; write it

        lda     #8              ; 8 bytes to send
        sta     cntr            ; save it
wrcglp: ldhx    addrhi          ; get data address
        lda     ,x              ; get the data to send
        aix     #1              ; increment address
        sthx    addrhi          ; save addr for later
        jsr     wrdat           ; send it
        dec     cntr            ; decrement byte count
        bne     wrcglp          ; do all eight
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      wrline
; Called By:    wrmess
; Calls:        wrdat (write data byte to LCD port)
; Gazintas:     addrhi,addrlo
; Gazoutas:     none
; Function:     This code will write 20 ascii characters to the LCD data port.
;               The address for these characters are already stored in locations
;               addrhi and addrlo.
; -----------------------------------------------------------------------------
wrline: lda     #20             ; twenty bytes per line
        sta     chrcnt          ; save this
wrllup: ldhx    addrhi          ; get the address for a character
        lda     ,x              ; get a byte to send
        aix     #1              ; increment pointer
        sthx    addrhi          ; save address for next round

        bsr     wrdat           ; write it
        dec     chrcnt          ; decrement character counter
        bne     wrllup          ; do this 20 times
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      wrstr, wrdats
; Called By:    stime
; Calls:        wrcmd (write a command byte to the LCD)
;               wrdat (write a data byte to the LCD)
; Gazintas:     (HX) is the message to write
; Gazoutas:     none
; Function:     This routine will write out a string starting at (HX) and
;               terminated with an ASCII NULL.  The first byte is the command
;               byte telling the LCD module where to put the string.
;               wrdats skips the initial command byte.
; -----------------------------------------------------------------------------
wrstr:  lda     ,x              ; get the command
        pshx                    ; save X
        bsr     wrcmd           ; send as a command
        pulx                    ; get X back

        aix     #1              ; point to the first data byte
wrdats:                         ; jump in here to skip the command byte        
wrslp:  lda     ,x              ; get it
        beq     wrsex           ; if NULL, exit
        stx     temp2           ; save pointer
        bsr     wrdat           ; otherwise send it
        ldx     temp2           ; get pointer back
        aix     #1              ; and point to next char
        bra     wrslp           ; do this until NULL

wrsex:  rts                     ; done!

        page
; -----------------------------------------------------------------------------
; Routine:      wrnstr
; Called By:    ssens
; Calls:        wrdat (write a data byte to the LCD)
; Gazintas:     (HX) is the message to write
;               A = number of characters to write
; Gazoutas:     none
; Function:     This routine will write out a string starting at (HX) and
;               the number of bytes in the A register.
; -----------------------------------------------------------------------------
wrnstr: psha                    ; save character count

wrnlp:  lda     ,x              ; get it
        stx     temp2           ; save pointer
        bsr     wrdat           ; otherwise send it
        ldx     temp2           ; get pointer back
        aix     #1              ; and point to next char

        pula                    ; get byte count back
        deca                    ; decrement it
        beq     wrnsex          ; if done, exit
        psha                    ; otherwise save count again
        bra     wrnlp           ; do this until NULL

wrnsex:  rts                     ; done!

        page
; -----------------------------------------------------------------------------
; Routine:      wrmstr
; Called By:    setup
; Calls:        wrcmd (write a command byte to the LCD)
;               wrdat (write a data byte to the LCD)
; Gazintas:     (HX) is the message to write
; Gazoutas:     none
; Function:     This routine will write out a string starting at (HX) and
;               terminated with an ASCII NULL.  This string always starts
;               at the upper-left corner and can span up to two lines.
; -----------------------------------------------------------------------------
wrmstr: pshx                    ; save X
        lda     #$80            ; top left corner
        bsr     wrcmd           ; send as a command
        pulx                    ; get X back
        clr     temp2           ; reset chracter counter

wrmslp: lda     ,x              ; get it
        beq     wrmsex          ; if NULL, exit
        pshx                    ; save X
        bsr     wrdat           ; otherwise send it
        pulx                    ; Restore X
        aix     #1              ; and point to next char
        inc     temp2           ; increment chracter counter
        lda     temp2           ; get it
        cmp     #20             ; see if EOL
        bne     wrmslp          ; do this until NULL

        pshx                    ; save X
        lda     #$c0            ; Line 2
        bsr     wrcmd           ; send as a command
        pulx                    ; get X back
        bra     wrmslp          ; and continue on

wrmsex: rts                     ; done!

        page
; -----------------------------------------------------------------------------
; Routine:      wrdat
; Called By:    losta routines
; Calls:        None
; Gazintas:     A: byte to write
; Gazoutas:     None
; Function:     This code will write a byte to the data register of the LCD
;               The byte to write is in the A register.  This routine breaks
;               up the 8 bits of data into two 4-bit chunks and writes them
;               out one at a time.  The byte sent is a data byte.
; -----------------------------------------------------------------------------
wrdat:  bsr     ckrdy           ; wait for LCD to be ready
        tax                     ; save data to send in X

        lda     ddra            ; get dir regs
        ora     #$fe            ; all bits ex 0 to ouputs
        sta     ddra            ; do that

        txa                     ; get data back
        and     #$f0            ; MSB only
        ora     #$02            ; RS=1,RW=0,Strobe=0
        sta     porta           ; assert MSB plus controls
        nop                     ; waste a bit of time
        bset    3,porta         ; assert the strobe
        nsa                     ; waste some time
        nsa                     ; waste some time
        bclr    3,porta         ; unassert the strobe

        txa                     ; get back data to write
        nsa                     ; put LSN in MSN
        and     #$f0            ; MSB only
        ora     #$02            ; RS=1,RW=0,Strobe=0
        sta     porta           ; assert MSB plus controls
        nop                     ; waste a bit of time
        bset    3,porta         ; assert the strobe
        nsa                     ; waste some time
        nsa                     ; waste some time
        bclr    3,porta         ; unassert the strobe

        rts                     ; all done!

        page
; -----------------------------------------------------------------------------
; Routine:      wrcmd
; Called By:    losta routines
; Calls:        None
; Gazintas:     A: byte to write
; Gazoutas:     None
; Function:     This code will write a byte to the data register of the LCD
;               The byte to write is in the A register.  This routine breaks
;               up the 8 bits of data into two 4-bit chunks and writes them
;               out one at a time.  The byte sent is a command byte
; -----------------------------------------------------------------------------
wrcmd:  bsr     ckrdy           ; wait for LCD to be ready
        tax                     ; save data to send in X

        lda     ddra            ; get dir regs
        ora     #$fe            ; all bits ex 0 to ouputs
        sta     ddra            ; do that

        txa                     ; get data back
        and     #$f0            ; MSB only
        ora     #$00            ; RS=0,RW=0,Strobe=0 (I know, useless)
        sta     porta           ; assert MSB plus controls
        nop                     ; wait a bit
        bset    3,porta         ; assert the strobe
        nsa                     ; waste some time
        nsa                     ; waste some time
        bclr    3,porta         ; unassert the strobe

        txa                     ; get back data to write
        nsa                     ; put LSN in MSN
        and     #$f0            ; MSB only
        ora     #$00            ; RS=0,RW=0,Strobe=0 (I know, useless)
        sta     porta           ; assert MSB plus controls
        nop                     ; wait a bit
        bset    3,porta         ; assert the strobe
        nsa                     ; waste some time
        nsa                     ; waste some time
        bclr    3,porta         ; unassert the strobe

        rts                     ; all done!

        page
; -----------------------------------------------------------------------------
; Routine:      wrini
; Called By:    losta routines
; Calls:        None
; Gazintas:     A: byte to write
; Gazoutas:     None
; Function:     This code will write a byte to the command register of the LCD
;               The byte to write is in the A register.  This routine breaks
;               up the 8 bits of data into two 4-bit chunks and writes them
;               out one at a time.  The byte written is a command byte, but
;               instead of checking to see if LCD controller is ready first,
;               I just send the command without asking and then delay the
;               maximum amount before returning.  This is needed for the first
;               initialization bytes before we can count on the ready bit
;               being correct.
; -----------------------------------------------------------------------------
wrini:  tax                     ; save value to write

        lda     ddra            ; get dir regs
        ora     #$fe            ; all bits ex 0 to ouputs
        sta     ddra            ; do that

        txa                     ; get value back
        and     #$f0            ; MSB only
        ora     #$00            ; RS=0,RW=0,Strobe=0 (I know, useless)
        sta     porta           ; assert MSB plus controls
        bset    3,porta         ; assert the strobe
        nsa                     ; waste some time
        nsa                     ; waste some time
        bclr    3,porta         ; unassert the strobe

        lda     #5              ; 5 miliseconds
        jsr     waitms          ; wait it
        rts                     ; all done!
        page

; -----------------------------------------------------------------------------
; Routine:      ckrdy
; Called By:    wrdat,wrcmd
; Calls:        None
; Gazintas:     None
; Gazoutas:     None
; Function:     Polls the LCD ready bit until the LCD says it is ready for
;               the next command/data byte.
; -----------------------------------------------------------------------------
ckrdy:  sta     temp            ; save a
ckrdy2: clr     rdy2            ; clear 2nd try ready flag

rdyagn: lda     ddra            ; get dir regs
        and     #$0f            ; bits 4:7 inputs
        ora     #$0e            ; bits 3:1 outputs
        sta     ddra            ; do that

        lda     #$04            ; RS=0,RW=1,Strobe=0
        sta     porta           ; write it
        bset    3,porta         ; assert the strobe
        nsa                     ; waste some time
        nsa                     ; waste some time
        lda     porta           ; get data
        bclr    3,porta         ; unassert the strobe

        bset    3,porta         ; assert the strobe for LSB (ignore data)
        nsa                     ; waste some time
        nsa                     ; waste some time
        bclr    3,porta         ; unassert the strobe

        and     #$80            ; get busy flag only
        bne     ckrdy2          ; wait for it to be ready

        lda     rdy2            ; get 2nd try flag
        bne     ready           ; if not zero, really ready

        inc     rdy2            ; otherwise make flag non-zero
        bra     rdyagn          ; and try again

ready:  lda     temp            ; get a back
        rts                     ; all done!

