        page
******************************************************
** Copyright 2003 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         timirq.asm
; Function:     Timer interrupt service handler
; Routines:     timirq: general timer interrupt handler
; -----------------------------------------------------------------------------

;       The timer interrupt routine.
;       This routine makes sure that it gets activated every 100ms
;       using the hardware timer for accuracy.  Every 100ms it
;       simply increments the timer compare register for 100ms
;       later and incrments the ruftim counter.
;       For the 6808 in regular mode, the timer tick is 2us.
;       100ms is a count of 50,000 or $c350.
;       The OCR registers can be read in any order, but must be written
;       MSB first.
;
;       For 6808 debug mode the bus clock is 2.4576MHz, so the count value
;       for 0.1 seconds is 15360 or $3c00
;
;       Here is where we scan the keypad too.  We read the buttons
;       every 100ms.  Since we sample data instead of looking for
;       edges, we don't need to do debouncing.
;
;       Here we also increment our real-time clock.  We have nine
;       counters to keep track of: year, month, day, UTC hours,
;       local bours, hours, minutes, seconds, and
;       sub-seconds (tenths).  We wiggle the output compare every
;       second as a heartbeat.  We also do not increment the time
;       when the rtchld semaphore is set.  It is set when we are
;       messing with setting the time.
;
;       An additional feature included here is clearing out the 24-hour
;       hi/lo values whenever the local time rolls over midnight.
;
;       Get the impression more and more stuff gets added to this routine???
;
;       This routine now also serves as the dispatcher for all the data
;       gathering and reporting routines.  See the comment header above
;       timdun for details.
;
;       Even though this is essentially a single routine, each of the functions
;       is pretty unique and will get a seperate header.
;
        page
; -----------------------------------------------------------------------------
; Routine:      timrtn (basics)
; Called By:    Timer Interrupt
; Calls:        None
; Gazintas:     None
; Gazoutas:     ruftim (rough time, increments by 1 every 100ms)
;               curkey (Currently pressed key)
; Function:     This is the first part of the timer interrupt.  The basics
;               done here include setting up the timer to interrupt me again
;               100ms later.
;               Second is to increment the rough timer value called ruftim.
;               This is used in several places where some long timeout values
;               are needed and this counter is used as a reference.
;               Finally I scan the keyboard here.  Since the keyboard gets
;               scanned only once every 100ms, there is little need for
;               debouncing.  Sorta cheating, but not really.
; -----------------------------------------------------------------------------


timrtn:;  mov     #$86,portb      ; say we're in the IRQ
        pshh                    ; save H
        lda     t1sc0           ; need to read first to clear later
        lda     t1cntl          ; get current ocr LSB
$IF DEBUG
        add     #$00            ; add 15,360 LSB
$ELSEIF
        add     #$50            ; add 50,000 LSB
$ENDIF
        tax                     ; hold on to that answer for a bit
        lda     t1cnth          ; get current MSB
$IF DEBUG
        adc     #$3c            ; add 15,360 mSB
$ELSEIF
        adc     #$c3            ; add 50,000 msb
$ENDIF
        sta     t1ch0h          ; store MSB
        stx     t1ch0l          ; store LSB

        inc     ruftim          ; increment rough time counter
        lda     cdflag          ; get current/daily alternating flag
        inca                    ; increment it
        sta     cdflag          ; save result

        lda     #$58            ; LED off value
        sta     t1sc0           ; write it

timkey: lda     portb           ; get the key bits
        and     #$78            ; look only at bits 6-3
        lsra                    ; now 5-2
        lsra                    ; now 4-1
        lsra                    ; now 3-0
        tax                     ; make an index
        clrh                    ; make sure H is 0
        lda     keytbl,x        ; get keycode
        sta     curkey          ; store it

        page
; -----------------------------------------------------------------------------
; Routine:      timrtn (real-time-clock section)
; Called By:    Timer Interrupt
; Calls:        mthmax - figgers last day of specified month)
; Gazintas:     rtchld (0=running, 1=hold)
; Gazoutas:     ssecs,secs,mins,lhrs,hrs,day,month,year (The time)
;               temphi,templo (Daily high and low temperature)
; Function:     This is the real-time clock support code.  Basically everything
;               gets incremente by a 10th of a second for each interrupt.
;               If rtchld is asserted, then this code gets skipped.  It is
;               asserted whenever the time setting routine is running.
;               Theoretically this bugger is Y2K-ready.
;               This also happens to be the convenient place to reset the
;               daily min and max temperature values when midnight local time
;               rolls around.
; -----------------------------------------------------------------------------
timrtc: jsr     mxiqen          ; conditionally re-enable interrupts
        tst     rtchld          ; see if RTC on hold
        beq     timrun          ; if 0, do the time thing
        jmp     timdun          ; if non-0, skip the time thing

timrun: inc     ssecs           ; increment sub-seconds
        lda     ssecs           ; get incremented value
        cmp     #10             ; see if 10 10ths
        blo     timdun          ; if not, done
        clr     ssecs           ; otherwise clear sub-seconds

        lda     #$5c            ; LED on value
        sta     t1sc0           ; write it

        inc     secs            ; increment seconds
        lda     secs            ; get incremented value
        cmp     #60             ; see if 60 seconds
        blo     timdun          ; if not, done
        clr     secs            ; otherwise clear seconds

        inc     mins            ; increment minutes
        lda     mins            ; get incremented value
        cmp     #60             ; see if 60 minutes
        blo     timdun          ; if not, done
        clr     mins            ; otherwise clear minutes

        lda     #1              ; not 0
        sta     hrroll          ; flag that hours have rolled

        inc     lhrs            ; increment local hours
        lda     lhrs            ; get incremented value
        cmp     #24             ; see if 24 hours
        blo     timutc          ; if not, done with local, on to UTC
        clr     lhrs            ; otherwise clear hours

        lda     temphi          ; get current high
        sta     yesthi          ; make it yesterdays
        lda     templo          ; get current low
        sta     yestlo          ; make it yesterdays
        ldhx    todygs          ; get todays max gust
        sthx    yestgs          ; store it
        ldhx    rancnt          ; get current rain count
        sthx    ranref          ; referece for new day
        ldhx    rantdy          ; todays rainfall
        sthx    ranyst          ; becomes yesterdays

        lda     #$ff            ; no min/max yet special flag
        sta     temphi          ; make it so
        sta     templo          ; clear low temp too
        sta     dtcal           ; say we want to re-read the time.
        clr     todygs          ; clear todays gust
        clr     todygs+1        ; MSB too

timutc: inc     hrs             ; increment hours
        lda     hrs             ; get incremented value
        cmp     #24             ; see if 24 hours
        blo     timdun          ; if not, done with UTC, on to local
        clr     hrs             ; otherwise clear hours

        inc     day             ; increment day-of-month
        jsr     mthmax          ; get month max value in A

        cmp     day             ; compare to current day count
        bhs     timdun          ; if days-in-month higher, then no action
        lda     #1              ; new day is 1st of month
        sta     day             ; remember that

        ldhx    rancnt          ; get the rain reference value
        sthx    ranmth          ; make sure default is a sane value

        inc     month           ; increment month
        lda     month           ; get incremented value
        cmp     #13             ; see if December
        blo     timdun          ; if not, done
        lda     #1              ; new month is January
        sta     month           ; remember that

        inc     year            ; increment year
        lda     year            ; get incremented value
        cmp     #100            ; see if 100 year (Y2K!)
        blo     timdun          ; if not, done
        clr     year            ; otherwise clear year to 00

        page
; -----------------------------------------------------------------------------
; Routine:      timrtn (samping routine dispatcher)
; Called By:    Timer interrupt
; Calls:        Lots (see notes below)
; Gazintas:     wxhold (0=get data, 1=don't)
; Gazoutas:     Lots (all weather data!)
; Function:     (see big comment below)
; -----------------------------------------------------------------------------

*       Now that we are done with most of the major housekeeping stuff that
*       gets done one every single interrupt, time to do the data gathering
*       that gets staged on 1/10th second intervals.  This is the major code
*       that gets introduced in the 1.04 release.
*
*       The basic idea here is to take the data gathering and statistics
*       gathering and do it all in interrupt service routines so the mainline
*       code can go do its UI stuff without worrying about the data being
*       gathered.  It takes a pretty long time to gather all the data, as of
*       this release, it takes up to 200ms of communications and a 600ms
*       pause for the temperature A/D to do its thing.  To not mess up the
*       UI by taking a big chunk of time, the data gathering is being broken
*       down into a bunch of small steps that are executed on each 100ms timer
*       tick.  The hope is that each part will take less than 10ms, so it won't
*       mess up the UI at all.
*
*       The task to run is determined by the irqseq counter that increments by
*       one every timer tick (100ms).  When it reaches 49 (5.00 seconds), it
*       gets reset to 0 and the sampling process starts all over again.  Here
*       are the assignments on a timer tick (irqseq) basis:
*
*          00: Temperature.... Start conversion
*       01-07: Temperature.... Wait for conversion to complete
*          08: Temperature.... Get and convert data
*          09: Wind Speed..... Get and convert data
*          10: Wind Direction. Open channel
*          11: Wind Direction. Get sample 1
*          12: Wind Direction. Get sample 2
*          13: Wind Direction. Close channel
*          14: Rain........... Get data
*          15: Humidity....... Conversions 1/2
*          16: Humidity....... Conversions 2/2
*          17: Humidity....... compute and average
*          18: Barometer...... Get Data
*          19: Barometer...... convert data
*          20: Statistics..... Cumulative stuff
*          21: T238 A/D work.. 1-Wire and Power voltage conversions
*          22: UART message... part 1 (if applicable)
*          23: UART message... part 2 (if applicable)
*          24: UART message... part 3 (if applicable)
*          25: UART message... part 4 (if applicable)
*          26: UART message... part 5 (if applicable)
*          27: Flag data available to main loop
*       28-48: Idle
*          49: Reset irqseq to 0

timdun: jsr     mxtirq          ; see if the MX-614 transmitter needs attention

        tst     wxhold          ; see if we are holding off doing WX samples
        beq     timseq          ; if not, go do the sequencing
        jmp     irqdun          ; if so, skip all this stuff

timseq: lda     irqseq          ; get sequence value
        cmp     #49             ; see if at (or greater than) max
        bcc     timrol          ; if so, roll irqseq back
        cmp     #28             ; see if idle states (28-48)
        bcc     seqdun          ; if so, go straight to done

        lsla                    ; multiply index by two
        tax                     ; make an index
        clrh                    ; MSB=0
        lda     jmptab,x        ; get target address MSB
        psha                    ; push it
        lda     jmptab+1,x      ; get target address LSB
        psha                    ; push it
        pulx                    ; pop LSB into X
        pulh                    ; pop MSB into H
        jsr     ,x              ; go to routine
        bra     seqdun          ; finished, wrap up IRQ

jmptab: fdb     iqtmp1          ; 00: set up temperature  ( 50ms)
        fdb     iqtmpw          ; 01: Wait for conversion (150ms)
        fdb     iqtmpw          ; 02: Wait for conversion (250ms)
        fdb     iqtmpw          ; 03: Wait for conversion (350ms)
        fdb     iqtmpw          ; 04: Wait for conversion (450ms)
        fdb     iqtmpw          ; 05: Wait for conversion (550ms)
        fdb     iqtmpw          ; 06: Wait for conversion (650ms)
        fdb     iqtmpw          ; 07: Wait for conversion (750ms)
        fdb     iqtmp2          ; 08: Get and process temperature data
        fdb     iqwspd          ; 09: Get and process wind speed
        fdb     iqdir1          ; 10: Open wind direction channel
        fdb     iqdir2          ; 11: Get wind direction sample 1
        fdb     iqdir3          ; 12: Get wind direction sample 2
        fdb     iqdir4          ; 13: Close wind direction channel
        fdb     iqrain          ; 14: Get rain data
        fdb     iqhum1          ; 15: Humidity conversions (1/2)
        fdb     iqhum2          ; 16: Humidity conversions (2/2)
        fdb     iqhum3          ; 17: Humidity compute value
        fdb     iqbar           ; 19: Get Barometer sensor output
        fdb     iqbar1          ; 20: Convert Barometer sensor output to B.P in InHg/Mb
        fdb     iqstat          ; 18: do cumulative statistics
        fdb     iqothr          ; 21: do infrequent stuff
        fdb     iqurt1          ; 22: UART message, part 1 (date/time)
        fdb     iqurt2          ; 23: UART message, part 2 (position)
        fdb     iqurt3          ; 24: UART message, part 3 (data, part 1)
        fdb     iqurt4          ; 25: UART message, part 4 (data, part 2)
        fdb     iqurt5          ; 26: UART message, part 5 (terminator)
        fdb     iqdun           ; 27: all done flag data avail

timrol: bsr     iqroll          ; go to the rollover code
        bra     irqdun          ; all finished

seqdun: inc     irqseq          ; increment sequence tracker

irqdun:; mov     #$06,portb      ; say we're out of the IRQ
        pulh                    ; restore H
        rti                     ; all done with interrupt

        page

iqstat: ;18: tally cumulative statistics
        jsr     tmpstt          ; do temperature stats
        jsr     spdstt          ; do wind speed stats
        jsr     dirstt          ; do wind direction stats
        jsr     ranstt          ; do rain stats
        jsr     humstt          ; do humidity stats
        jsr     barstt          ; do barometer stats
        rts                     ; all done

iqothr: ; 19 Do infrequent stuff
;       ; This is for items that do not need to happen every 5 seconds.
;       ; I use the cumcnt (5-minute) counter to decide what stuff to
;       ; do.
;       ; EVEN: (every 10 seconds) digitize 1-wire bus voltage
;       ;  ODD: (for now) digitize battery voltage
;       ; If dtcal is set (at midnight), do a time cal)
;
        lda     dtcal           ; get the time/date cal flag
        beq     iqondt          ; if not set, normal business

;        jsr     ck1994          ; go re-read the time (if no 1994, no harm)
        lda     #0              ; 0...
        sta     dtcal           ; clear the recal flag
        bra     iqoend          ; do just this

iqondt: lda     cumcnt          ; get our 5-minute counter
        and     #$01            ; look at LSB only
        bne     iqodd           ; go do odd stuff
        jsr     a2dc1w          ; go convert 1-wire voltage
        bra     iqoend          ; all done

iqodd:  jsr     a2dcbt          ; go convert battery voltage

iqoend: rts                     ; all done

iqdun:  ;15: all done, flag data avail
        inc     newdat          ; set the new data flag
        rts

iqroll: ; 49: At end of sequence, start all over again.
        inc     cumcnt          ; increment our sample counter
        lda     cumcnt          ; get result
        cmp     #57             ; 10 seconds before transmitting
        beq     iqron           ; if so, turn transmitter on
        cmp     #2              ; 15 seconds after transmitting
        beq     iqrof           ; if so, turn transmitter off

        cmp     #60             ; see if 60 samples (5 minutes @ 5 secs)
        bne     iqrreg          ; if not, regular processing

        clr     cumcnt          ; if at end of roll, clear sample counter again
        jsr     newtmp          ; re-initialize temperature stats
        jsr     newspd          ; re-initialize wind speed stats
        jsr     newdir          ; re-initialize wind direction stats
        jsr     newran          ; do 5-minute rain stat processing
        jsr     newhum          ; re-initialize humidity stats
        jsr     newbar          ; re-initialize barometer stats/do 5 minute Bp stats
        bra     iqrreg          ; all done

iqron:  bset    0,portc         ; I/O line high to turn on transmitter
        bra     iqrreg          ; all done

iqrof:  bclr    0,portc         ; I/O line high to turn on transmitter
        bra     iqrreg          ; all done

iqrreg: clr     irqseq          ; start sampling sequence back at 0
        rts                     ; all done

