        page
******************************************************
** Copyright 2006 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         uart.asm
; Function:     The UART drivers and message generators.
; Routines:     iqurt1: Sends UART string 1 (date/time)
;               iqurt2: Sends UART string 2 (position)
;               iqurt3: Sends UART string 3 (data, part 1)
;               iqurt4: Sends UART string 4 (data, part 2)
;               iqurt5: Sends UART string 4 (terminator)
;               sbaud:  Sets up the baud rate depending on intended use
;               smsg:   Sends a RAM-based message. (currently unused)
;               srmsg:  Sends a ROM-based message.
;               irqmsg: Fires off an interrupt-based message
;               scitx:  UART transmit interrupt service routine
; -----------------------------------------------------------------------------
;
; The UART message is broken into three parts.  The first part is the UTC time,
; the second is an optional position string (5-min averaged only),
; and the third part is the actual weather data.  I break this into two
; parts to save on buffer size as well as breaking the message up into pieces
; that will take less than half the alloted 100ms to send out.  Messages are
; sent out via an interrupt-driven driver so the entire time needed to send
; the message is not hogged up by the interrupt service routine.
;
; The UART routines are called every 5-second sample.  For computer-based
; messaging, the data is sent out every sample.  For TNC messaging, the data
; is only sent out every 60th sample and contains averaged data instead of
; the "instantaneous" data.
;
; For Peet mode I use only the first two routines to queue up data, iqurt3-5 are
; not used.  See Peet.asm for details, that code is kept seperate
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      iqurt1
; Called By:    timrtn
; Calls:        bin2as (convert binary to ASCII)
;               irqmsg (enables IRQ-based UART message)
; Gazintas:     fumode (UART TNC/Computer/Peet mode)
;               cumcnt (Wx Sample counter)
;               month,day,hrs,mins (the current UTC time)
; Gazoutas:     msg (the time message)
;               msgptr (pointer to the message)
; Function:     This routine generates the time part of the weather data
;               message, sending the sequence _MMDDHHMM (month, day, hours,
;               minutes) to the UART.  In the case of Computer mode, this
;               data is sent every sample.  For TNC mode, this string is
;               only sent every 60th (#59) sample when sending out the
;               averaged data.
;
;               If we are in the positioned format, the time/data string
;               is a bit different, instead it is @DDHHMMz.  Dunno why,
;               they just made it different.  Rather than two independent
;               routines for each slightly different format, I simply do
;               the positionless format, then edit it if we are positioned.
;
;               In Peet mode we just bypass this all and go to ptmsg1.
;
;               NOTE:  THE MESSAGE QUEUED UP HERE **MUST** TAKE LESS THAN
;               100MS AS IQURT2 WILL BE QUEUING UP DATA 100MS FROM NOW!!
; -----------------------------------------------------------------------------

iqurt1: lda     fumode          ; see what UART mode we are in
        cmp     #3              ; see if Peet mode
        bne     iqunp1          ; if not, process as normal
        jmp     ptmsg1          ; otherwise do the Peet thing.

iqunp1: cmp     #2              ; See if computer mode
        beq     iqurgo          ; if Computer mode, always send time
        lda     cumcnt          ; if TNC modes, (0,1,4) get cumulative sample counter
        cmp     #59             ; see if after last (60th) sample
        bne     iqur1x          ; if not, don't send time

iqurgo: jsr     mxinit          ; clear out the MX-614 buffer

        lda     #1              ; not 0
        sta     zfill           ; want 0's filled in

        lda     #'_'            ; time/weather identifier
        sta     msg             ; save marker

        lda     month           ; get the month
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+1           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+2           ; store it

        lda     day             ; get the day
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+3           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+4           ; store it

        lda     hrs             ; get the hour
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+5           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+6           ; store it

        lda     mins            ; get the minutes
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+7           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+8           ; store it

        clr     msg+9           ; mark end of string
        lda     fipon           ; see if positioned string
        bne     iqur1t          ; if not, all done

        lda     fumode          ; get the current mode
        cmp     #2              ; see if computer mode
        beq     iqur1t          ; if computer mode, ignore position flag

;       For the positioned format we replace the '_' with '@', delete the
;       month from the string, and put a 'z' on the end.

        mov     #'@',msg        ; the @ prefix

        clrx                    ; need to copy incrementing
        clrh                    ; MSB=0
iqu1cl: lda     msg+3,x         ; source byte
        sta     msg+1,x         ; destination byte
        incx                    ; next
        cpx     #6              ; see if done
        bne     iqu1cl          ; loop till done

        mov     #'z',msg+7      ; time flag
        clr     msg+8           ; end of time message

iqur1t: jsr     irqmsg          ; send it
iqur1x: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqurt2
; Called By:    timrnt
; Calls:        ptmsg2 (peet message 2nd half
;               irqmsg (send interrupt-based message)
;               irqmsg (enables IRQ-based UART message)
; Gazintas:     fumode (Computer/TNC/Peet mode flag)
;               cumcnt (Wx data sample counter)
;               fipon (send position string flag)
;               fiposn (position string)
; Gazoutas:     msg (the message to send)
;               msgptr (pointer to above message)
; Function:     This routine generates the position string if it is called
;
;               TNC modes, it is not generated in the computer or Peet
;               mode.  The position data is flagged by the fipon flag and
;               the user-specified 18-byte string is stored at fiposn.
;
;               In Peet mode we just bypass this all and go to ptmsg2.
; -----------------------------------------------------------------------------
iqurt2: lda     fumode          ; see what UART mode we are in
        cmp     #3              ; see if Peet mode
        bne     iqunp2          ; if not, process as normal
        jmp     ptmsg2          ; otherwise do the Peet thing.

iqunp2: cmp     #02             ; see if computer mode
        beq     iqu2ex          ; if Computer mode, no position possible
        lda     cumcnt          ; if TNC mode, get cumulative sample counter
        cmp     #59             ; see if after last (60th) sample
        bne     iqu2ex          ; if not, don't send data
        lda     fipon           ; get "send position" flag
        bne     iqu2ex          ; if non-zero, don't send position

;       We get here if we wanna send the string. Note that I copy all 18
;       bytes plus the "send message" flag.  It is 00 by definition and also
;       ends up being my end of string flag.

        ldx     #18             ; wanna copy 19 bytes
        clrh                    ; msb=0
iqu2cp: lda     fiposn,x        ; copy position string from flash
        sta     msg,x           ; write to message buffer
        decx                    ; copy all bytes
        bpl     iqu2cp          ; loop till done

        jsr     irqmsg          ; go send the message
iqu2ex: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqurt3
; Called By:    timrnt
; Calls:        tncmsg (generates TNC message with averaged data)
;               cmpmsg (generates computer message with instant data)
;               irqmsg (enables IRQ-based UART message)
; Gazintas:     fumode (Computer/TNC/Peet mode flag)
;               cumcnt (Wx data sample counter)
; Gazoutas:     msg (the message to send)
;               msgptr (pointer to above message)
; Function:     This is the third part of the UART message to be sent.  For
;               computer messages, instant data is sent every sample, for
;               TNC operations, averaged data is sent every 60th sample.
;               The tncmsg or cmpmsg routines generate the string, then this
;               code enables the SCI transmit ISR to actuall send the data
;
;               If we are in positioned mode, the TNC string gets altered a
;               bit before going out.  See tncmsg for details.
;
;               In Peet mode we just bypass this all and go to ptmsg3.
; -----------------------------------------------------------------------------
iqurt3: lda     fumode          ; see what UART mode we are in
        cmp     #3              ; see if Peet mode
        bne     iqunp3          ; if not, process as normal
        jmp     ptmsg3          ; otherwise do the Peet thing.

iqunp3: cmp     #02             ; see if computer mode
        beq     iqucmp          ; if Computer mode, go send data
        lda     cumcnt          ; if TNC mode, get cumulative sample counter
        cmp     #59             ; see if after last (60th) sample
        bne     iqur3x          ; if not, don't send data
        jsr     tncmsg          ; otherwise generate TNC message
        bra     iqusnd          ; and send it

iqucmp: jsr     cmpmsg          ; do the computer message
iqusnd: jsr     irqmsg          ; send the message
iqur3x: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqurt4
; Called By:    timrnt
; Calls:        tncmsg2
;               cmpmes2
; Gazintas:     None
; Gazoutas:     None
; Function:     This is the forth part of the UART message to be sent.  For
;               computer messages, instant data is sent every sample, for
;               TNC operations, averaged data is sent every 60th sample.
;               The tncmsg2 or cmpmsg2 routines generate the string, then this
;               code enables the SCI transmit ISR to actuall send the data
;
;               This routine is ignored in Peet mode, the peet routines have
;               already called the code to transmit the data.
;
; -----------------------------------------------------------------------------
iqurt4: lda     fumode          ; see what UART mode we are in
        cmp     #03             ; see if Peet mode
        beq     iqu4x           ; if peet mode, do nuthin
        cmp     #02             ; see if computer mode
        beq     iqcmp2          ; if Computer mode, go send data
        lda     cumcnt          ; if TNC mode, get cumulative sample counter
        cmp     #59             ; see if after last (60th) sample
        bne     iqu4x           ; if not, don't send data
        jsr     tncms2          ; otherwise generate TNC message
        bra     iqsnd2          ; and send it

iqcmp2: jsr     cmpms2          ; do the computer message
iqsnd2: jsr     irqmsg          ; send the message
iqu4x:  rts                     ; do nothing

        page
; -----------------------------------------------------------------------------
; Routine:      iqurt5
; Called By:    timrnt
; Calls:        dbadd  (converts debug data to ascii)
;               irqmsg (sends message to UART)
; Gazintas:     fumode (Computer/TNC/Peet mode flag)
;               cumcnt (IRQ sequence counter)
;               dbptr  (debug array pointer)
; Gazoutas:     msg (the message to send)
; Function:     This is the fifth part of the UART message to be sent.  Under
;               Normal curcumstances, this routine just sends
;               the CR or CR/LF depending on the mode.  If there is debug data
;               present, this routine will append it to the end of the TNC
;               message before sending the CR to tell the TNC to transmit the
;               data.  The purpose of this code is to help me locate problems
;               with weather stations that are not mine by puting some debug
;               data in the TNC string.
;
;               This routine is ignored in Peet mode, the peet routines have
;               already called the code to transmit the data.
; -----------------------------------------------------------------------------
iqurt5: lda     fumode          ; see what UART mode we are in
        cmp     #03             ; see if Peet mode
        beq     iq4ndx          ; if peet mode, do nuthin
        cmp     #02             ; see if computer mode
        beq     iq4msg          ; if Computer mode, go send data
        lda     cumcnt          ; if TNC mode, get cumulative sample counter
        cmp     #59             ; see if after last (50th) sample
        beq     iq4msg          ; if so, send data

iq4ndx: jmp     iqur4x          ; long jump to exit

iq4msg: lda     dbptr           ; get debug pointer
        bne     iq4hdb          ; if debug data, go render it

        lda     hrroll          ; see if hours has rolled since last xmission
        beq     iq4ndb          ; if not, no more possibilities

        lda     #$01            ; 0x01 is error counter flag
        sta     dbbuf+0         ; store it
        lda     errtmp          ; Temperature error counter
        sta     dbbuf+1         ; store it
        lda     errspd          ; Wind Speed error counter
        sta     dbbuf+2         ; store it
        lda     errdir          ; Wind direction error counter
        sta     dbbuf+3         ; store it
        lda     errran          ; Rain counter error counter
        sta     dbbuf+4         ; store it
        lda     errhum          ; Humidity sensor error counter
        sta     dbbuf+5         ; store it
        lda     a2d1w           ; 1-wire bus voltage
        sta     dbbuf+6         ; store it
        lda     a2dbty          ; battery voltage
        sta     dbbuf+7         ; store it
        lda     #$08            ; 8 bytes in the buffer
        sta     dbptr           ; store it
        lda     #$00            ; clear the hour-rolled flag
        sta     hrroll          ; store it
        bra     iq4hdb          ; go render it

iq4ndb: clrh                    ; get HX to 0 in case no mesage
        clrx                    ; X too
        bra     iq4nda          ; go send just CR to terminate data

iq4hdb:
; DEBUG CODE: If the debug message is of type 3, add the wind speed sample count
        lda     dbbuf+0         ; get the mesage type
        cmp     #$03            ; see if type 03
        bne     iq4n03          ; if not, normal rendering

        lda     wsamp           ; get the wind speed sample count
        sta     dbbuf+7         ; save it as part of the message
; END OF DEBUG CODE

iq4n03: mov     #' ',msg        ; a space from the e1w
        clr     temp            ; our byte counter
iq4dlp: bsr     dbadd           ; write a byte dbbuf(temp) to msg(temp*2+1,temp*2+2)
        inc     temp            ; point to next
        lda     temp            ; get it
        cmp     dbptr           ; see if done
        bne     iq4dlp          ; if not, on to next byte

        lda     dbptr           ; get debug pointer
        asla                    ; times two
        add     #1              ; add one
        tax                     ; make an index
        clrh                    ; clear msb (hx) is offset to end of string

        clra                    ; 0...
        sta     dbptr           ; mark message as sent

iq4nda: lda     #$0d            ; a carriage return
        sta     msg,x           ; save it
        lda     #$0a            ; a line feed
        sta     msg+1,x         ; save it
        lda     #$00            ; end-of-string
        sta     msg+2,x         ; save it
        lda     fumode          ; get the TNC mode
        cmp     #1              ; see if TNC-LTP
        beq     iq4smg          ; if so, send message as-is
        clr     msg+1,x         ; otherwise kill the LF and send

iq4smg: jsr     irqmsg          ; send the message
        jsr     mxterm          ; terminate the MX-614 message
        jsr     mxgo            ; fire up MX-614
iqur4x: rts                     ; all done


dbadd:  clrh                    ; MSB=0
        ldx     temp            ; HX is offset in dbbuf
        lda     dbbuf,x         ; get the byte
        pshx                    ; save X for a bit
        nsa                     ; Want MSB only
        and     #$0f            ; got it
        tax                     ; now an index
        lda     hextb,x         ; get ASCII value
        pulx                    ; get index back
        aslx                    ; times 2
        incx                    ; plus one
        sta     msg,x           ; store it
        incx                    ; point to next byte
        pshx                    ; save pointer
        ldx     temp            ; HX is offset in dbbuf
        lda     dbbuf,x         ; get the byte
        and     #$0f            ; LSB only
        tax                     ; make an index
        lda     hextb,x         ; get ASCII value
        pulx                    ; get pointer back
        sta     msg,x           ; write the byte
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      sbaud
; Called By:    initnc
; Calls:        None
; Gazintas:     fbaud:  0=2400
;                       1=4800
;                       2=9600
;                       others=mode default
;               fumode: mode to figger default
;                       0=TNC Direct    (4800)
;                       1=TNC KPC3+ LTP (2400)
;                       2=computer      (9600)
;                       3=Peet          (2400)
; Gazoutas:     None
; Function:     This is the routine to set up the UART and baud rate.  Depending
;               on if we are in TNC or computer mode, we want 4800 and 9600 baud
;               respectively.  Further complicating things a bit is that we need
;               different constants depending on if we are in regular or debug
;               mode.
;
;               KPC3+ LTP Mode changes by N1BQ (6/10/00) (changed Baud rate from
;               4800 to 2400 baud
; -----------------------------------------------------------------------------
sbaud:
        lda     #$40            ; SCI on, 8 bits, no parity, no weird stuff
        sta     scc1            ; make it so
        lda     #$0c            ; Rx,Tx on, no IRQs, no breaks
        sta     scc2            ; make it so
        lda     #$00            ; No 9th bit, DMA, or noise processing.
        sta     scc3            ; make it so

; Check explicit stuff first.
        lda     fbaud           ; get explicit baud rate
        beq     sb24            ; if 0, do 2400 baud
        cmp     #1              ; see if 1
        beq     sb48            ; if so, do 4800 baud
        cmp     #2              ; see if 2
        beq     sb96            ; if so, 9600 baud

; If none of the above, do default based on fumode

        lda     fumode          ; see if computer or TNC mode
        beq     sb48            ; 0=TNC-dir =4800
        cmp     #1              ; 1=TNC-LTP =2400
        beq     sb24            ;   set 2400
        cmp     #2              ; 2=computer=9600
        beq     sb96            ;   set 9600
        cmp     #3              ; 3=Peet    =2400 (fall through)
        beq     sb24            ;   set 2400
sb_bad:
;       Code should go here to point back at UART Setup for bad mode value,
;       but probably would not be needed if setup menus done correctly
;
sb24:
$IF DEBUG
        lda     #$06            ; 2400 baud assuming 2.???MHz bus clock
$ELSEIF
        lda     #$34            ; 2400 baud assuming 8MHz bus clock
$ENDIF
        bra     sbwex           ; go set the baudrate and exit

sb96:
$IF DEBUG
        lda     #$04            ; 9600 baud assuming 2.???MHz bus clock
$ELSEIF
        lda     #$32            ; 9600 baud assuming 8MHz bus clock
$ENDIF
        bra     sbwex           ; go set the baudrate and exit

sb48:
$IF DEBUG
        lda     #$05            ; 4800 baud assuming 2.???MHz bus clock
$ELSEIF
        lda     #$33            ; 4800 baud assuming 8MHz bus clock
$ENDIF
sbwex:  sta     scbr            ; make it so
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      srmsg
; Called By:    initnc
; Calls:        waits
; Gazintas:     h:x address of string in ROM to send
; Gazoutas:     The text out the UART.
; Function:     This is the routine that sends a ROM message out the UART.
;               This routine sends the null-terminated string starting at h:x,
;               waits a half-second, then returns.  This is pretty unique to
;               TNC messgaes because of the extra half-second at the end for
;               the TNC to "digest" the message.
; -----------------------------------------------------------------------------
srmsg:  lda     ,x              ; get a byte of data
        beq     srmex           ; if 0, all done

        sta     scdr            ; send it on it's way
srmwlp: brclr   6,scs1,srmwlp   ; wait for Tx buffer to empty

        aix     #1              ; increment byte pointer
        bra     srmsg           ; do bytes until a null character

srmex:  lda     #5              ; 1/2 second wait
        jsr     waits           ; wait it
        rts                     ; all done sending message

        page
; -----------------------------------------------------------------------------
; Routine:      tncmsg
; Called By:    iqurt2
; Calls:        avgdir (get averaged wind direction)
;               avgtmp (get averaged temperature)
; Gazintas:     The accumulated weather stats
; Gazoutas:     (msg) The averaged weather string
; Function:     This is the routine that creates the second half of a TNC
;               message consisting of the averaged data.  The first half has
;               already been created and delivered.  The parts of this
;               message are the underlined part below:
;               _MMDDHHMMcxxxsxxxgxxxtxxxPxxxrxxxhxxbxxxxxe1w
;                        ====================
;               cxxx is wind direction (degrees)
;               sxxx is wind speed (MPH)
;               gxxx is wind gust (MPH)
;               txxx is temperature (Farenheight)
;               Pxxx is rain since midnight (1/100ths of an inch)
;               rxxx is rain past hour (1/100ths of an inch)
;               hxx  is relative humidity (%)
;               bxxxxx is barometric pressure in Mb and tenth Mb
;               e1w  is my station identifer (Embedded, 1-wire)
;
;               In positioned mode, the initial wind direction and speed get
;               changed to _xxx/xxx, probably for historical reasons.  In
;               support of this I just do the entire string for positionless
;               mode and then edit it at the end.
; -----------------------------------------------------------------------------
tncmsg: lda     #1              ; not 0
        sta     zfill           ; want 0's filled in

        lda     #'c'            ; wind direction indicator
        sta     msg             ; save marker

        jsr     avgdir          ; convert averaged wind direction to degrees

        lda     atemp           ; wind direction byte 1
        sta     msg+1           ; store it
        lda     atemp+1         ; wind direction byte 2
        sta     msg+2           ; store it
        lda     atemp+2         ; wind direction byte 3
        sta     msg+3           ; store it

        lda     #'s'            ; Wind speed next
        sta     msg+4           ; save marker

        lda     ptavws          ; get quotient MSB
        sta     cnv2a           ; store for conversion
        lda     ptavws+1        ; get quotient LSB
        sta     cnv2a+1         ; store it
        jsr     piwind          ; convert to ASCII

        lda     atemp           ; Hundreds digit
        sta     msg+5           ; store it
        lda     atemp+1         ; speed byte 1
        sta     msg+6           ; store it
        lda     atemp+2         ; speed byte 2
        sta     msg+7           ; store it

        lda     #'g'            ; Wind gust next
        sta     msg+8           ; save marker

        jsr     ldxams          ; get 10-minute max wind in X:A
        stx     cnv2a           ; save MSB
        sta     cnv2a+1         ; save LSB
        jsr     piwind          ; convert 5-sec value to MPH

        lda     #'0'            ; MSB is always 0
        sta     msg+9           ; store it
        lda     atemp+1         ; speed byte 1
        sta     msg+10          ; store it
        lda     atemp+2         ; speed byte 2
        sta     msg+11          ; store it

        lda     #'t'            ; Temperature next
        sta     msg+12          ; save marker

        jsr     avgtmp          ; go figger the average temperature

        lda     atemp           ; temperature byte 1
        sta     msg+13          ; store it
        lda     atemp+1         ; temperature byte 2
        sta     msg+14          ; store it
        lda     atemp+2         ; temperature byte 3
        sta     msg+15          ; store it

        clr     msg+16          ; assume end-of-message for now

        ldhx    #msg+16         ; next location to put text
        lda     rirgot          ; see if we got a rain sensor
        bne     tncnrn          ; if not, go to terminator

        lda     #'P'            ; Rain-since-midnight identifier
        sta     msg+16          ; store it
        ldhx    rantdy          ; get todays rain count
        sthx    divdnd          ; store for conversion
        jsr     pirain          ; convert to ASCII
        lda     atemp           ; get 100's
        sta     msg+17          ; store it
        lda     atemp+1         ; get 10's
        sta     msg+18          ; store it
        lda     atemp+2         ; get 1's
        sta     msg+19          ; store it

        clr     msg+20          ; end of string

tncnrn: lda     fipon           ; see if we are positioned string format
        bne     tncrtx          ; if not, do nuthin

        mov     #'_',msg        ; change first byte to underscore
        mov     #'/',msg+4      ; change byte between wind course and speed to slash

tncrtx: rts                     ; all done.

; -----------------------------------------------------------------------------
; Routine:      tncms2
; Called By:    iqurt2
; Calls:        avgbar
; Gazintas:     The accumulated weather stats
; Gazoutas:     (msg) The averaged weather string
; Function:     This is the routine that creates the second half of a TNC
;               message consisting of the averaged data.  The first half has
;               already been created and delivered.  The parts of this
;               message are the underlined part below:
;               _MMDDHHMMcxxxsxxxgxxxtxxxPxxxrxxxhxxbxxxxxe1w
;                                            ================
;               cxxx is wind direction (degrees)
;               sxxx is wind speed (MPH)
;               gxxx is wind gust (MPH)
;               txxx is temperature (Farenheight)
;               Pxxx is rain since midnight (1/100ths of an inch)
;               rxxx is rain past hour (1/100ths of an inch)
;               hxx  is relative humidity (%)
;               bxxxxx is barometric pressure in Mb and tenth Mb
;               e1w  is my station identifer (Embedded, 1-wire)
;
;
; -----------------------------------------------------------------------------
tncms2: clr     msg             ; assume no message for now
        ldhx    #msg            ; next location to put text
        sthx    msg+30          ; save it

        lda     rirgot          ; see if we got a rain sensor
        bne     tncnr2          ; if not, go to terminator

        lda     #'r'            ; Rain-last-hour identifier
        sta     msg+0           ; store it

        ldhx    ran1hr          ; get rain past hour
        sthx    divdnd          ; save it for conversion

        brclr   7,errran,tmsnrn ; if rain OK, convert
        clr     divdnd          ; otherwise clear to 0 if reading error
        clr     divdnd+1        ; clear LSB too

tmsnrn: jsr     pirain          ; convert to ASCII
        lda     atemp           ; get 100's
        sta     msg+1           ; store it
        lda     atemp+1         ; get 10's
        sta     msg+2           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+3           ; store it

        ldhx    #msg+4          ; new place for terminating string
        sthx    msg+30          ; save it.

tncnr2: lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     tncnhu          ; if not $26, no sensor installed

        lda     hmtot           ; see if there is at least one sample
        beq     tncnhu          ; if not, skip humidity entry

        jsr     avghum          ; go compute average humidity
        ldhx    msg+30          ; get place to put it

        lda     #'h'            ; h is humidity indicator
        sta     0,x             ; store it
        lda     atemp+1         ; get 10's digit
        sta     1,x             ; store it
        lda     atemp+2         ; get 1's digit
        sta     2,x             ; store it

        aix     #3              ; move pointer ahead three
        sthx    msg+30          ; save it.

tncnhu: lda     fibar           ; get barometerID first byte
        cmp     #$26            ; see if family code exists
        bne     tnced           ; if not $26, no sensor installed

        lda     bacnt           ; get barometer counter
        cmp     #$00            ; compare to 0
        beq     tnced           ; leave if no samples

        jsr     avgbar          ; go get average and convert to ASCII
        ldhx    msg+30          ; get the offset back

tncbar: lda     #'b'            ; b is the Barometeric Pressure indicator
        sta     0,x             ; store it

        lda     atemp           ; get 1000 digit
        sta     1,x             ; store it
        lda     atemp+1         ; get 100's digit
        sta     2,x             ; store it
        lda     atemp+2         ; get 10's digit
        sta     3,x             ; store it
        lda     atemp+3         ; get 1' digit
        sta     4,x             ; store it
        lda     temp            ; get 0.1;s digit
        sta     5,x             ; store it

        aix     #6              ; move msg pointer up 6
        sthx    msg+30          ; save it

tnced:  ldhx    msg+30          ; get message pointer
        lda     #'e'            ; my self-declared type of e for embedded
        sta     0,x             ; save marker

        lda     #'1'            ; my self-declared sub-type of 1w for 1-wire
        sta     1,x             ; save marker

        lda     #'w'            ; my self-declared sub-type of 1w for 1-wire
        sta     2,x             ; save marker

        clr     3,x             ; end of string

        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      cmpmsg
; Called By:    iqurt2
; Calls:        inidot (initializes text to dashes if bad data)
;               piwind (convert current wind speed to ASCII)
;               pitemp (convert current temperature to ASCII)
; Gazintas:     The accumulated weather stats
; Gazoutas:     (msg) The averaged weather string
; Function:     This is the routine that creates the second half of a TNC
;               message consisting of the averaged data.  The first half has
;               already been created and delivered.  The parts of this
;               message are the underlined part below:
;               _MMDDHHMMcxxxsxxxtxxxPxxxrxxxhxxbxxxxxe1w
;                        ====================
;               cxxx is wind direction (degrees)
;               sxxx is wind speed (MPH)
;               txxx is temperature (Farenheight)
;               Pxxx is rain since midnight (1/100ths of an inch)
;               rxxx is rain past hour (1/100ths of an inch)
;               hxx  is relative humidity (%)
;               bxxxxx is Barometric Pressure (xxxx.x Mb)
;               e1w  is my station identifer (Embedded, 1-wire)
; -----------------------------------------------------------------------------
cmpmsg: lda     #1              ; not 0
        sta     zfill           ; want 0's filled in

        lda     #'c'            ; wind direction indicator
        sta     msg             ; save marker

        jsr     inidot          ; initialize atemp to '...'
        brset   7,errdir,cmwer  ; if errors, leave dashes

        lda     #2              ; want zero-filled numeric wind direction
        jsr     piwdir          ; go do it and put in atemp

cmwer:  lda     atemp           ; wind direction byte 1
        sta     msg+1           ; store it
        lda     atemp+1         ; wind direction byte 2
        sta     msg+2           ; store it
        lda     atemp+2         ; wind direction byte 3
        sta     msg+3           ; store it

        lda     #'s'            ; Wind speed next
        sta     msg+4           ; save marker

        jsr     inidot          ; initialize atemp to '...'
        lda     errspd          ; get error flags and counter
        and     #$c0            ; get only error flags
        bne     cmser           ; if errors, leave dashes

        lda     winspd          ; get current wind speed MSB
        sta     cnv2a           ; conversion buffer
        lda     winspd+1        ; get cyrrebt wind speed LSB
        sta     cnv2a+1         ; conversion buffer
        jsr     piwind          ; convert 5-sec value to MPH

cmser:  lda     atemp           ; speed byte 0
        sta     msg+5           ; store it
        lda     atemp+1         ; speed byte 1
        sta     msg+6           ; store it
        lda     atemp+2         ; speed byte 2
        sta     msg+7           ; store it

        lda     #'t'            ; Temperature next
        sta     msg+8           ; save marker

        jsr     inidot          ; initialize atemp to '...'
        brset   7,errdd,cmter   ; if errors, leave dashes

        lda     ddtemp          ; get current temperature
        sub     #110            ; convert to two's comp
        sta     cnv2a           ; place to get converted
        jsr     pitemp          ; convert current temp to degrees farenheit

cmter:  lda     atemp           ; temperature byte 1
        sta     msg+9           ; store it
        lda     atemp+1         ; temperature byte 2
        sta     msg+10          ; store it
        lda     atemp+2         ; temperature byte 3
        sta     msg+11          ; store it

        ldhx    #msg+12         ; next location to put text
        lda     rirgot          ; see if we got a rain sensor
        bne     cmpnrn          ; if not, go to terminator

        lda     #'P'            ; Rain-since-midnight identifier
        sta     msg+12          ; store it
        ldhx    rantdy          ; get todays rain count
        sthx    divdnd          ; store for conversion
        jsr     pirain          ; convert to ASCII
        lda     atemp           ; get 100's
        sta     msg+13          ; store it
        lda     atemp+1         ; get 10's
        sta     msg+14          ; store it
        lda     atemp+2         ; get 1's
        sta     msg+15          ; store it

        lda     #'r'            ; Rain-last-hour identifier
        sta     msg+16          ; store it

        ldhx    ran1hr          ; get rain past hour
        sthx    divdnd          ; save it for conversion

        brclr   7,errran,cmsnrn ; if rain OK, convert
        clr     divdnd          ; otherwise clear to 0 if reading error
        clr     divdnd+1        ; clear LSB too

cmsnrn: jsr     pirain          ; convert to ASCII
        lda     atemp           ; get 100's
        sta     msg+17          ; store it
        lda     atemp+1         ; get 10's
        sta     msg+18          ; store it
        lda     atemp+2         ; get 1's
        sta     msg+19          ; store it

        ldhx    #msg+20         ; new place for terminating string

cmpnrn: clr     0,x             ; end of string

        rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      cmpms2
; Called By:    iqurt2
; Calls:        inidot (initializes text to dashes if bad data)
;               tcba0a (converts Bavalm to ASCII
; Gazintas:     The accumulated weather stats
; Gazoutas:     (msg) The averaged weather string
; Function:     This is the routine that creates the second half of a TNC
;               message consisting of the averaged data.  The first half has
;               already been created and delivered.  The parts of this
;               message are the underlined part below:
;               _MMDDHHMMcxxxsxxxtxxxPxxxrxxxhxxbxxxxxe1w
;                                            ============
;               cxxx is wind direction (degrees)
;               sxxx is wind speed (MPH)
;               txxx is temperature (Farenheight)
;               Pxxx is rain since midnight (1/100ths of an inch)
;               rxxx is rain past hour (1/100ths of an inch)
;               hxx  is relative humidity (%)
;               bxxxxx is Barometric Pressure (xxxx.x Mb)
;               e1w  is my station identifer (Embedded, 1-wire)
; -----------------------------------------------------------------------------
cmpms2: ldhx    #msg            ; start of message
        sthx    msg+30          ; save it

        lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     cmpnhu          ; if not $26, no sensor installed

;       Put in dots initially for the humidity value.  If we have a read
;       error, I leave the dots.  I print the computed value regardless of
;       if the over-range value is set.  The only value I don't print is
;       a value of 100%, this is limited to 99%.

        ldhx    msg+30          ; get index
        lda     #'h'            ; h is humidity indicator
        sta     0,x             ; store it
        lda     #'.'            ; a dot
        sta     1,x             ; store it in 10's
        sta     2,x             ; store it in 1's

        lda     errhum          ; get humidity error counter
        and     #$80            ; error flag
        bne     cmpher          ; if error, leave dots

        lda     hmval           ; get current humidity
        and     #$7f            ; clear excess flag
        cmp     #100            ; see if exactly 100%
        bne     cmpnrx          ; if not, all done
        lda     #99             ; limit to 99%

cmpnrx: sta     divdnd+1        ; place to convert
        clr     divdnd          ; make sure MSB is 0
        jsr     bin2as          ; convert to ASCII

        ldhx    msg+30          ; get index back
        lda     atemp+1         ; get 10's digit
        sta     1,x             ; store it
        lda     atemp+2         ; get 1's digit
        sta     2,x             ; store it

cmpher: aix     #3              ; bump up message index
        sthx    msg+30          ; save result

        ; On to Barometric Pressure

cmpnhu: lda     fibar           ; get barometerID first byte
        cmp     #$26            ; see if family code exists
        bne     cmped           ; if not $26, no sensor installed

;       Put in dots initially for the Baro. Pressure value.  If we have a read
;       error, I leave the dots

        ldhx    msg+30          ; get message pointer
        lda     #'b'            ; b is the Barometeric Pressure indicator
        sta     0,x             ; store it
        lda     #'.'            ; a dot
        sta     1,x             ; store in 1000's
        sta     2,x             ; store in 100's
        sta     3,x             ; store it in 10's
        sta     4,x             ; store it in 1's
        sta     5,x             ; store in 0.1's

        lda     errhum          ; get error counter
        bmi     cmphue          ; if error, leave dots.

        lda     bavalm          ; get Baro. Press in metric(MSB)
        sta     divdnd          ; store it
        lda     bavalm+1        ; get Baro. Press in metricLSB)
        sta     divdnd+1        ; store it

        jsr     tcba0a          ; go convert to ASCII

        ldhx    msg+30          ; get message pointer
        lda     atemp           ; get 1000 digit
        sta     1,x             ; store it
        lda     atemp+1         ; get 100's digit
        sta     2,x             ; store it
        lda     atemp+2         ; get 10's digit
        sta     3,x             ; store it
        lda     atemp+3         ; get 1' digit
        sta     4,x             ; store it
        lda     temp            ; get 0.1;s digit
        sta     5,x             ; store it

cmphue: aix     #6              ; bump message index by BP data field
        sthx    msg+30          ; save index

cmped:  ldhx    msg+30          ; get index back.
        lda     #'e'            ; my self-declared type of e for embedded
        sta     0,x             ; save marker

        lda     #'1'            ; my self-declared sub-type of 1w for 1-wire
        sta     1,x             ; save marker

        lda     #'w'            ; my self-declared sub-type of 1w for 1-wire
        sta     2,x             ; save marker

        clr     3,x             ; end of string

        rts                     ; all done
        page

; -----------------------------------------------------------------------------
; Routine:      inidot
; Called By:    cmpmsg
; Calls:        None
; Gazintas:     None
; Gazoutas:     atemp (initialized to three dashes)
; Function:     This is simply a quick and dirty routine to initialize atemp
;               to dots in case the real data is not valid.  Used a lot, so
;               it made sense to routine-ize it.
; -----------------------------------------------------------------------------
inidot: lda     #'.'            ; a dot
        sta     atemp           ; store it
        sta     atemp+1         ; again
        sta     atemp+2         ; and again
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      irqmsg
; Called By:    iqurt1, iqurt2, iqurt3, iqurt4, ptmsg1, ptmsg2.
; Calls:        None
; Gazintas:     None
; Gazoutas:     msgptr (pointer to UART message)
; Function:     This routine fires off an intrrupt-based UART message.
;               The message is in (msg) and is up to 31 characters plus a
;               null terminating chracter.  This routine sets up the pointer
;               to the message for the irq routine (scitx) and then enables
;               the interrupts so the message can be sent out in the
;               background.
;
;               NOTE:  The way this routine is used, messages get queued up
;               every 100ms timer tick.  As implemented, one string must be
;               fully sent before the next can be queued up.  Given a little
;               margin and stuff, it is best to not queue up more than 80ms
;               of data each 100ms interrupt.  Listed are the max amount of
;               data you should queue up if you expect to add more data on the
;               next interrupt.
;               Baud Rate Max chars
;               ========= =========
;               1200      9 chars  (not currently used)
;               2400      19 chars
;               4800      38 chars (buffer is 31 max)
;               9600      76 chars (buffer is 31 max)
; -----------------------------------------------------------------------------
irqmsg: jsr     mxqdat          ; queue up the data for the MX-614

        mov     #msg,msgptr     ; reset message pointer to message buffer
        bset    6,scc2          ; enable SCI transmit interrupts
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      scitx
; Called By:    SCI Interrupt request
; Calls:        None
; Gazintas:     msgptr (pointer to message buffer)
;               msg (the message to send)
; Gazoutas:     Character out the UART
; Function:     This is the SCI Transmit interrupt service routine.  During
;               normal data transmission operations, UART transmissions are
;               interrupt-driven.  To transmit data, the buffer MSG is stuffed
;               with null-terminated message to send and then the SCI transmit
;               interrupt enable is asserted.   Once enabled, an interrupt will
;               be pending and the transmission will start.  At the completion
;               of the message (reading null byte), this routine will disable
;               itself to prevent further interrupts until the message
;               generator re-enables the interrupt again.  This routine is not
;               intended to handle new messages while a message is being
;               transmitted, that is, the current message is assumed to be
;               completed before a new message is started.  The message
;               generator needs to initialize the message pointer (msgptr) to
;               the start of the message before enabling interrupts.
; -----------------------------------------------------------------------------
scitx:  pshh                    ; save H
        clrh                    ; clear it
        ldx     scs1            ; dummy read to clear IRQ
        ldx     msgptr          ; get the pointer to the byte to send
        lda     ,x              ; get data
        bne     sctxdx          ; if not null, send the data!

sctxdn: bclr    6,scc2          ; no more data, clear TX interrpt enable
        bra     sctxex          ; all done

sctxdx: mov     x+,scdr         ; if not, write the byte to send
        stx     msgptr          ; save incremented pointer
sctxex: pulh                    ; restore H
        rti                     ; all done
