        page
******************************************************
** Copyright 2006 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         wdir.asm
; Function:     Wind direction sampling, statistics, and averaging routines
; Routines:     inidir: Initialize wind direction variables
;               newdir: 5-minute statistic re-initialization
;               ROUTINE DALLAS SENSOR FUNCTION        AAG SENSOR FUNCTION
;               ======= ============================= =========================
;               iqdir1: Open wind direction channel   Fire off A/D conversions
;               iqdir2: Get wind direction sample 1   Gather/determine wind dir
;               iqdir3: Get wind direction sample 2   Unused.
;               iqdir4: Close wind direction channel  Unused.
;               dirstt: Do Wind direction stats
;               piwdir: Figure out instant wind direction (interrupt)
;               pmwdir: Figure out instant wind direction (mainline)
;               avgdir: Compute 5-minute average wind direction
;               rddir:  Read the direction (not interrupt driven)
; -----------------------------------------------------------------------------
;
;       These are the routines to figger out the wind direction.  They are pretty
;       nasty.  The way this works is that there are eight reed switches in
;       a circle above the weather vane.  The weather vane is hooked up to a
;       magnet that will close one (directly over) or two adjacent (in between)
;       of the reed switches.  The in between feature gives us double the
;       number of data points, 22.5 degrees instead of only 45.  For each
;       reed switch there is a DS2401 silicon ID.  When the reed switch is
;       closed the ID chip is hooked up to the bus and can be read.  The
;       way to find out the direction is to go out and see what devices are
;       present on the bus using a scan process.  Practically, there can
;       only be one or two devices on the bus, no more no less.
;
;       For each ID (which are listed in incrementing order of degrees) we
;       assign a number from 0-7 and take two readings.  When the weather
;       vane is exactly on the same heading as a sensor, we will get two
;       readings of the same value.  When the weather vane is in between
;       two sensors, we will get one reading of each.  When we add the two
;       numbers together, we effectively get 16 possible readings since all
;       odd totals will be the in-betwen values and even numbers will be when
;       we get the same number twice.
;       There is an exception to this simple math.  If one reading is 7 and
;       the other is 0, the total needs to be 15, not 7.
;
;       If an error is encountered while trying to get the wind sensor ID, we
;       try up to #tries times before giving up.
;
;       AAG WIND DIRECTION FUNCTIONALITY
;       =======================================================================
;       The folks at AAG decided to make some changes to the wind direction
;       sensor arrangement using some newer parts to reduce circuit complexity.
;       The arrangement and function of the eight switches remains the same.
;       The difference is that they run these switches into a matrix with
;       resistors and come up with four input voltages that will be either
;       high, middle, or ground depending on which combination of reed switches
;       are closed.  You'll need to look at the schematic for details, but here
;       is the table you end up with based on wind direction number.  For each
;       channel I do a 3-bit conversion and based on that value decide if the
;       voltage is high, middle, or low.
;
;       DIRECTION  SWITCHES CLOSED A/D A/D A/D A/D
;                  Closed          A   B   C   D
;       ========== =============== === === === ===
;       1          S1              .   .   M   .     . = High Voltage
;       2          S1 S2           .   M   M   .     M = Middle Voltage
;       3          S2              .   M   .   .     L = Low Voltage
;       4          S2 S3           M   M   .   .
;       5          S3              M   .   .   .     A/D value meaning
;       6          S3 S4           M   .   .   L     =================
;       7          S4              .   .   .   L     0 = Low
;       8          S4 S5           .   .   L   L     1 = Low
;       9          S5              .   .   L   .     2 = Mid
;       10         S5 S6           .   L   L   .     3 = Mid
;       11         S6              .   L   .   .     4 = Mid
;       12         S6 S7           L   L   .   .     5 = Mid
;       13         S7              L   .   .   .     6 = High
;       14         S7 S8           L   .   .   M     7 = High
;       15         S8              .   .   .   M
;       16         S8 S1           .   .   M   M
;

; -----------------------------------------------------------------------------
; Routine:      calwdr
; Called By:    ssens
; Calls:        isrch (find ID)
;               wrmess (write LCD message to LCD)
;               waits (wait)
;               cnvid (convert ID to ASCII)
;               wrstr (write LCD null-terminated string)
;               wrcmd (write LCD command byte)
;               wrdat (write LCD data byte)
;               wfnkey (wait for next key press)
;               rdf64 (read 64-byte Flash block
;               wrf64 (write 64-byte flash block)
;               appera (erase flash block)
; Gazintas:     None
; Gazoutas:     Flash location fitmp is programmed
; Function:     This is the temperature sensor finder program.  For this code
;               I explain each code block as I go.
; -----------------------------------------------------------------------------
calwdr: lda     #1              ; message #1 (Warning message)
        jsr     wrmess          ; write it
        lda     #40             ; four second wait
        jsr     waits           ; wait it

        lda     #15             ; message #15 (Ask if OK to procede)
        jsr     wrmess          ; write it
ssq1:   jsr     wfnkey          ; wait for next keypress
        cmp     #ksel           ; see if select button (continue)
        beq     sscont          ; if so, on to calibrating

sslex:  rts                     ; if not, return doing nothing

;       Next get the wind direction switch ID.  Two possibilities exist.  The
;       first is the regular Dallas wind direction setup which has a DS2407
;       switch and eight DS2401 ID chips.  Unfortunately, a second DS2407 can
;       exist if the rain guage is plugged in.  If so, I have a problem
;       knowing which is which. Choices are to either ask the user to unplug
;       the rain gauge or to show them the IDs of both 2407s and let them
;       pick which is the correct one.
;       The second option is if the AAG "3rd generation" wind direction
;       sensor is installed.  For now it has a unique sensor ID, for the
;       DS2450, so if that ID exists, we are all set without any
;       sensor complications.
;
;       I check for the DS2450 first as it is the easiest.  If it exists,
;       all my sensor selections are done.

sscont: lda     #$20            ; DS2450 family ID
        jsr     isrch           ; go find an ID, put into mndray
        tst     mndray          ; see if success
        bne     calwd1          ; if so, go program it

        lda     #$12            ; family code for the DS2407 switch
        jsr     i2srch          ; go see if there are any
        tsta                    ; set flags on A (just in case)
        beq     calhns          ; if none, neither AAG or Dallas, error
        deca                    ; see if only one sensor
        beq     calwd1          ; go there if so
        bra     calwd2          ; if 2, got some decisions to make

calhns: lda     #46             ; No Sensor message
        jsr     wrmess          ; write it
        lda     #40             ; 4 seconds
        jsr     waits           ; wait it
        rts                     ; and exit

calwd2: lda     #10             ; big message #10 (resolution choices)
        jsr     wr4mes          ; write it

calwdq: jsr     wfnkey          ; wait for next keypress
        cmp     #kup            ; see if up key (remove a sensor)
        beq     calwdr          ; if so, to see if one sensor now
        cmp     #kdown          ; see if down button (user pick)
        beq     calwdp          ; if so, let user pick
        bra     calwdq          ; if neither, try again

;       We get here if the user chose to look at both IDs and decide which is
;       the real wind sensor.  This is now a shared routine.

calwdp: jsr     i2pick          ; let user pick, answer in (mndray)

calwd1: ldhx    #fpage0         ; the page to read
        jsr     rdf64           ; read it

        ldhx    mndray          ; get bytes 0,1
        sthx    msg+8           ; store them
        ldhx    mndray+2        ; get bytes 0,1
        sthx    msg+10          ; store them
        ldhx    mndray+4        ; get bytes 0,1
        sthx    msg+12          ; store them
        ldhx    mndray+6        ; get bytes 0,1
        sthx    msg+14          ; store them

        sei                     ; interrupts off
        ldhx    #fpage0         ; point to whole page again
        jsr     appera          ; erase it
        ldhx    #fpage0         ; point to whole page again
        jsr     wrf64           ; program altered values
        cli                     ; interrupts back on

        jsr     iniaag          ; initialize AAG sensor (if it exists)
        bcs     calhns          ; if there and error initializing, say probs

;       Now that we have the main sensor type (AAG A/D or Dallas bus switch),
;       time to erase the sensor array.  I erase both Flash and the msg
;       buffer no matter which type of sensor I have so that the flash
;       contents will be at a predictable state.

        sei                     ; interrupts off
        ldhx    #fidir1         ; the page to erase
        jsr     appera          ; go erase it
        cli                     ; interrupts back on

sswdrt: ldhx    #63             ; want to clear 64 bytes
ssclrp: clr     msg,x           ; clear a byte
        decx                    ; decrement counter
        bpl     ssclrp          ; do all 64 bytes

;       We only need to do the rotation "discovery" process for the Dallas
;       wind direction sensor, if we have the AAG sensor, skip this part
;       and go straight to the place where we ask for North.

        lda     fisw            ; get direction sensor family ID
        cmp     #$20            ; see if AAG (DS2450)
        bne     sswnag          ; if not AAG, go read IDs

        jmp     sswaag          ; otherwise just to north cal

sswnag: lda     #16             ; message #16 (rotate slowly msg)
        jsr     wrmess          ; write it

        clr     cntr            ; new ID counter
        lda     #$3f            ; command to turn 2407 channel B on
        jsr     iswich          ; go do it

ssilp:  clr     flg12           ; ID hi/low flag set low (not important here)
        lda     #$01            ; direction sensor family ID
        jsr     isrch           ; go find an ID, put into mndray
        jsr     amatch          ; see if we match any existing IDs
        bcs     ssnew           ; no match, must be new entry
        inca                    ; increment it
        cmp     cntr            ; match had better be last ID read
        beq     ssilp           ; if so, continue normally
        jmp     sswdrt          ; if not, start over

ssnew:  lda     cntr            ; get current counter
        lsla                    ; local cntr ; 2 (A reg is still entry #)
        lsla                    ; local cntr ; 4
        lsla                    ; local cntr ; 8 (now offset to table)
        tax                     ; X is no offset into table
        clrh                    ; make sure H is 0

        lda     mndray          ; read id byte 0
        sta     msg+0,x         ; save it
        lda     mndray+1        ; read id byte 1
        sta     msg+1,x         ; store it
        lda     mndray+2        ; read id byte 2
        sta     msg+2,x         ; store it
        lda     mndray+3        ; read id byte 3
        sta     msg+3,x         ; store it
        lda     mndray+4        ; read id byte 4
        sta     msg+4,x         ; store it
        lda     mndray+5        ; read id byte 5
        sta     msg+5,x         ; store it
        lda     mndray+6        ; read id byte 6
        sta     msg+6,x         ; store it
        lda     mndray+7        ; read id byte 7
        sta     msg+7,x         ; store it

        inc     cntr            ; increment counter
        lda     cntr            ; get counter ID
        add     #$ca            ; LCD memory offset to display #
        jsr     wrcmd           ; write LCD address
        lda     cntr            ; get counter ID
        add     #'0'            ; Convert to ASCII digit 1-8
        jsr     wrdat           ; write LCD address

        lda     cntr            ; get our unique ID counter back
        cmp     #8              ; see if total is eight
        beq     sstep2          ; if eight, on to step 2
        jmp     ssilp           ; if not, go wait for next

sserr:  jmp     sswdrt          ; long jump to start over

sstep2: lda     #$c7            ; location for 2 in 2/3
        jsr     wrcmd           ; send location
        lda     #'2'            ; the 2 to write
        jsr     wrdat           ; send it.

        lda     #7              ; start with current counter value
        sta     cntr            ; make it current

ss2wt2: clr     flg12           ; ID hi/low flag set low (not important here)
        lda     #$01            ; direction sensor family ID
        jsr     isrch           ; go find an ID, put into mndray
        jsr     amatch          ; see if we match any existing IDs
        bcs     sserr           ; if no match, major error
        cmp     cntr            ; see if still current counter
        beq     ss2wt2          ; wait until new ID #
        sta     temp            ; save new value

        lda     cntr            ; get current value
        inca                    ; increment to expected value
        and     #$07            ; truncate things
        sta     cntr            ; save it

        lda     temp            ; get current compare back
        cmp     cntr            ; compare to expected new value
        bne     sserr           ; if not that, major error

        add     #$cb            ; LCD memory offset to display #
        jsr     wrcmd           ; write LCD address
        lda     #'-'            ; for phase 2, just write dashes
        jsr     wrdat           ; write LCD address

        lda     cntr            ; get counter value back
        cmp     #7              ; see if done yet
        bne     ss2wt2          ; if not, go get next ID

        lda     #7              ; start with current counter value
        sta     cntr            ; make it current

sstep3: lda     #$c7            ; location for 3 in 2/3
        jsr     wrcmd           ; send location
        lda     #'3'            ; the 2 to write
        jsr     wrdat           ; send it.

ss2wt3: clr     flg12           ; ID hi/low flag set low (not important here)
        lda     #$01            ; direction sensor family ID
        jsr     isrch           ; go find an ID, put into mndray
        jsr     amatch          ; see if we match any existing IDs
        bcs     sserr           ; if no match, major error
        cmp     cntr            ; see if still current counter
        beq     ss2wt3          ; wait until new ID #
        sta     temp            ; save new value

        lda     cntr            ; get current value
        inca                    ; increment to expected value
        and     #$07            ; truncate things
        sta     cntr            ; save it

        lda     temp            ; get current compare back
        cmp     cntr            ; compare to expected new value
        bne     sserr           ; if not that, major error

        add     #$cb            ; LCD memory offset to display #
        jsr     wrcmd           ; write LCD address
        lda     cntr            ; get counter value
        add     #'1'            ; for phase32, re-write count
        jsr     wrdat           ; write LCD address

        lda     cntr            ; get counter value back
        cmp     #7              ; see if done yet
        bne     ss2wt3          ; if not, go get next ID

        lda     #$7f            ; command to turn 2407 channel B off
        jsr     iswich          ; go do it

;       Now that we have a confirmed array of sensor IDs, time to write
;       them to flash.  This is only done for the Dallas sensor, for
;       the AAG sensor this got erased earlier and will stay erased.

        sei                     ; interrupts off
        ldhx    #fidir1         ; the page to write to
        jsr     wrf64           ; write all the IDs
        cli                     ; interrupts back on

;       Finally all that is left to do is ask where north is.  At this point
;       all we have is the eight sensors, but no real idea where anything
;       absolute is.  We simply ask the person to point north and press
;       select.  We read in the direction, two's complement it to make it
;       the correct offset and are done.

sswaag: lda     #17             ; message #17 ( point north & press select)
        jsr     wrmess          ; write it
ssq3:   jsr     wfnkey          ; wait for next keypress
        cmp     #ksel           ; see if select button (continue)
        bne     ssq3            ; wait until that is the key.

        jsr     rddir           ; go get the current direction
        lda     windir          ; get the result
        sub     fwdoff          ; undo any previous offset
        nega                    ; make an offset
        sta     rdtemp          ; save it for a bit

        ldhx    #fpage0         ; the page to read
        jsr     rdf64           ; read it

        lda     rdtemp          ; get offset back
        sta     msg+40          ; Place for the offset
        lda     #$ff            ; unprogrammed value
        sta     msg+41          ; clear the rest
        sta     msg+42          ; clear the rest
        sta     msg+43          ; clear the rest
        sta     msg+44          ; clear the rest
        sta     msg+45          ; clear the rest
        sta     msg+46          ; clear the rest
        sta     msg+47          ; clear the rest

        sei                     ; interrupts off
        ldhx    #fpage0         ; point to whole page again
        jsr     appera          ; erase it
        ldhx    #fpage0         ; point to whole page again
        jsr     wrf64           ; program altered values

        cli                     ; interrupts back on
        rts                     ; all done

;
; -----------------------------------------------------------------------------
; Routine:      inidir, newdir
; Called By:    wxgo,timrtn
; Calls:        None
; Gazintas:     None
; Gazoutas:     cumwdr
; Function:     First-time initialization and 5-minute re-initialization for
;               wind direction sampling are identical.  All we need to do is
;               clear out the 16-byte array of bytes that get incremented
;               by one for each wind direction sample that is taken.  Each
;               bin represents a 22.5 degree wide cone that the wind speed
;               was measured as pointing to.
; -----------------------------------------------------------------------------
inidir:
newdir: ldx     #15             ; want to clear 16 values
        clrh                    ; make sure MSB is 0
inidrl: clr     cumwdr,x        ; clear a byte
        decx                    ; decrement counter
        bpl     inidrl          ; clear all 15 bytes
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqdir1
; Called By:    timrtn
; Calls:        iswitch (open/close direction sensor switch)
; Gazintas:     None
; Gazoutas:     errdir (wind direction error flag/counter)
; Function:     Getting the wind direction is a four step process, this is the
;               first.  A feature of the 1-wire bus is that whenever a device
;               sees power for the first time, it asserts the bus low for a
;               time to tell the master that someone new showed up.  Cool if
;               it is the only device, but wreaks havok on any existing
;               communication.  Put eight of these on a weather vane that is
;               wafting in the breeze (literally), and you have a bad
;               situation.  The solution the Dallas folk came up with was to
;               put the DS2401s behind a programmable switch.  This way the
;               sensors are isolated behind the switch when they are not
;               needed and only connected when we need to do a wind speed
;               reading.  This first step calls the iswitch routine that
;               opens the channel.
;               Here is also where we clear any old error bits and if the
;               switch open command fails for some reason, set the error
;               flag.
; -----------------------------------------------------------------------------
;       For AAG mode I just go fire off the A/D converters
; -----------------------------------------------------------------------------

iqdir1: bclr    7,errdir        ; clear error flag for fresh start
        lda     fisw            ; get switch/ADC family ID
        cmp     #$20            ; see if AAG-based setup
        bne     iqdr1d          ; if not, exit
; -----------------------------------------------------------------------------
; AAG Support
; -----------------------------------------------------------------------------

        ldhx    #fisw           ; get the A/D chip address
        jsr     sndadr          ; send it
        jsr     crcini          ; clear out the CRC
        lda     #$3c            ; Convert command
        jsr     i_writ          ; send it
        lda     #$0f            ; convert all four channels
        jsr     i_writ          ; send it
        lda     #$55            ; initialize to all 0's
        jsr     i_writ          ; send it

        jsr     i_read          ; get CRC LSB
        jsr     i_read          ; get CRC MSB
        jsr     crcchk          ; See if we get the correct answer
        bne     iqdr1e          ; if not, set error flags

        mov     #$86,ddrb       ; want high power mode
        bra     iqdr1x          ; all done

; -----------------------------------------------------------------------------
; Original Dallas Support
; -----------------------------------------------------------------------------

iqdr1d: lda     #$3f            ; command to turn 2407 channel B on
        jsr     iswich          ; go do it
        beq     iqdr1x          ; Z set means success
iqdr1e: bset    7,errdir        ; and set error flag
        mov     #$ff,mndray     ; flag as serious error
iqdr1x: rts                     ; all done (errors or not)


; -----------------------------------------------------------------------------
; Routine:      iqdir2
; Called By:    timrtn
; Calls:        isrch (go find a sensor ID)
;               match (compare ID to table of IDs)
; Gazintas:     errdir (error flag/counter)
; Gazoutas:     windir (first wind direction sample)
; Internals:    flg12 (hi/low address search flag)
; Function:     Step two is to go get the first wind direction address out
;               there.  The isrch routine goes out and queries the bus for
;               a device with the proper ID.  See isrch for a description
;               of what flg12 does, it is VERY important in terms of what
;               it does and what it DOESN'T do.
;               After getting the first reading and converting it to a 0-7
;               number, we store it in windir.  If the match fails, we
;               just stuff in 0 and set the error flag.
;               Also, if we had errors opening the channel, there is no sense
;               in trying to read data, so this whole routine gets skipped.
;               I try up to three times got get an ID before giving up.  An
;               ID read takes about 20ms, so three would chew up 60ms of my
;               alloted 100ms before the next timer tick.
; -----------------------------------------------------------------------------
;       For AAG mode I read the A/D data
; -----------------------------------------------------------------------------
iqdir2: mov     #$80,ddrb       ; high power mode off no matter what
        brset   7,errdir,iqdr2x ; if errors, skip read

        lda     fisw            ; get switch/ADC family ID
        cmp     #$20            ; see if AAG-based setup
        bne     iqdr2d          ; if not, exit

; -----------------------------------------------------------------------------
; AAG Support
; -----------------------------------------------------------------------------

        ldhx    #fisw           ; get the A/D chip address
        jsr     sndadr          ; send it
        jsr     crcini          ; clear out the CRC
        lda     #$aa            ; Read memory command
        jsr     i_writ          ; send it
        lda     #$00            ; Address LSB
        jsr     i_writ          ; send it
        lda     #$00            ; Address MSB
        jsr     i_writ          ; send it

        jsr     i_read          ; Get Channel A LSB
        jsr     i_read          ; Get Channel A MSB
        sta     mndray          ; Store MSB
        jsr     i_read          ; Get Channel B LSB
        jsr     i_read          ; Get Channel B MSB
        sta     mndray+1        ; Store MSB
        jsr     i_read          ; Get Channel C LSB
        jsr     i_read          ; Get Channel C MSB
        sta     mndray+2        ; Store MSB
        jsr     i_read          ; Get Channel D LSB
        jsr     i_read          ; Get Channel D MSB
        sta     mndray+3        ; Store MSB

        jsr     i_read          ; get CRC LSB
        jsr     i_read          ; get CRC MSB
        jsr     crcchk          ; See if we get the correct answer
        bne     iqdr2e          ; if not, set error flags
        bra     iqdr2x          ; otherwise done

; -----------------------------------------------------------------------------
; Original Dallas Support
; -----------------------------------------------------------------------------

iqdr2d: lda     #3              ; three tries
        sta     atemp           ; remember that

iqd2r:  lda     #$01            ; family code for wind direction ibuttons.
        clr     flg12           ; this flag says ack 1st addr bit
        jsr     isrch           ; go find one.
        jsr     match           ; go get index
        bcc     rdds2           ; if match, go save result

        dec     atemp           ; decrement #tries counter
        bne     iqd2r           ; try up to three times

iqdr2e: bset    7,errdir        ; and set error flag
        mov     #$ff,mndray     ; flag as serious error
        clra                    ; for error, save 0
rdds2:  sta     windir          ; save result
iqdr2x: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqdir3
; Called By:    timrtn
; Calls:        isrch (go find a sensor ID)
;               match (compare ID to table of IDs)
; Gazintas:     errdir (error flag/counter)
; Gazoutas:     windir (combined wind direction sample)
; Internals:    flg12 (hi/low address search flag)
; Function:     Step 3 starts off pretty much the same as step 2.
;               After getting the second 0-7 sample, we add it to the previous
;               one for a total.  We need to convert the 0,7 and 7,0 sums to
;               15 to handle that rollover condition.  We detect this by
;               seeing if the total is 7 and the last sample was 0 or 7.  If
;               so, we add 8.
; -----------------------------------------------------------------------------
;       For AAG mode I take the raw readings and figger direction
; -----------------------------------------------------------------------------
iqdir3: brset   7,errdir,iqdr3x ; if errors, skip read

        lda     fisw            ; get switch/ADC family ID
        cmp     #$20            ; see if AAG-based setup
        bne     iqdr3d          ; if not, exit

; -----------------------------------------------------------------------------
; AAG Support
; -----------------------------------------------------------------------------
; To get direction I have a two-step process.  The first is to take the A/D
; values and convert them to a 2-bit value, 00 for low, 01 for middle, and
; 10 for high values.  I concatenate the four two-bit values together and
; then do a lookup to match the possibilities for the 16 directions.

        clr     windir          ; start off clean
        lda     mndray          ; get channel A (3 MSBs are data)
        sta     aagdbg          ; DEBUG for temporary debug screen
        jsr     iqdrbt          ; go process two bits
        lda     mndray+1        ; get channel B (3 MSBs are data)
        sta     aagdbg+1        ; DEBUG for temporary debug screen
        jsr     iqdrbt          ; go process two bits
        lda     mndray+2        ; get channel C (3 MSBs are data)
        sta     aagdbg+2        ; DEBUG for temporary debug screen
        jsr     iqdrbt          ; go process two bits
        lda     mndray+3        ; get channel D (3 MSBs are data)
        sta     aagdbg+3        ; DEBUG for temporary debug screen
        jsr     iqdrbt          ; go process two bits

        ldx     #15             ; max index for wind directions
        lda     windir          ; get the wind direction code
iq3dl:  cmp     aagdir,x        ; see if we have a match
        beq     iqd3hd          ; if so, sone
        decx                    ; decrement index
        bpl     iq3dl           ; loop till end (or match)
        bra     iqdr3e          ; if no match, error

iqd3hd: stx     windir          ; save pre-offset value
        bra     iqdr3x          ; fix offset and exit

iqdrbt: nsa                     ; twiddle mibbles
        rora                    ; 3 bits now LSBs
        and     #$07             ; value now 0-7
        tax                     ; make an index
        clrh                    ; want H 0...
        lda     aagvol,x        ; get Hi/Mid/Low flags
        rora                    ; shift MSB into carry
        rol     windir          ; put in wind direction
        rora                    ; shift LSB into carry
        rol     windir          ; put in wind direction
        rts                     ; done

; -----------------------------------------------------------------------------
; Original Dallas Support
; -----------------------------------------------------------------------------

iqdr3d: lda     #3              ; three tries
        sta     atemp           ; remember that

iqd3r:  mov     #1,flg12        ; non-0 flag says to ack 2nd addr bit.
        lda     #$01            ; family code for wind direction ibuttons.
        jsr     isrch           ; go find one.
        jsr     match           ; go get index
        bcc     rdds3           ; if match, go process

        dec     atemp           ; decrement #tries counter
        bne     iqd3r           ; try up to three times

iqdr3e: bset    7,errdir        ; and set error flag
        clr     windir          ; for error, return 0
        bra     iqdr3x          ; if we fail, abort

rdds3:  sta     temp            ; save result for a bit
        add     windir          ; add last result
        sta     windir          ; save total
        cmp     #7              ; see if total was 7 (special case)
        bne     iqdr3x          ; if not, all done
        lda     temp            ; get the second reading again
        beq     rdspec          ; if 0, the special case
        cmp     #7              ; see if second reading is 7 (special)
        bne     iqdr3x          ; if not, not special

rdspec: lda     windir          ; get direction
        add     #8              ; add eight
        sta     windir          ; save resultant direction

iqdr3x: lda     windir          ; get the direction back
        add     fwdoff          ; add the 0-offset
        and     #$0f            ; make sure its 0-15
        sta     windir          ; and save result
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqdir4
; Called By:    timrtn
; Calls:        iswitch (set direction switch setting)
; Gazintas:     none
; Gazoutas:     errdir
; Function:     This is the final step in the sampling process.
;               Here we close the channel.  This is attempted regardless of
;               any previous errors as closing the channel is extremely
;               important to keep down bus noise for future samples.  If
;               there were any errors during the four steps, here is where
;               we finally increment the error counter.
; -----------------------------------------------------------------------------
;       For AAG mode I make sure the data is valid and reprogram the DS2450
;       if not.
; -----------------------------------------------------------------------------

iqdir4: lda     fisw            ; get switch/ADC family ID
        cmp     #$20            ; see if AAG-based setup
        bne     iqdr4d          ; if not, exit checking for errors

; -----------------------------------------------------------------------------
; AAG Support:  Take the four readings from the A/D converter.  For all four
; of these readings, the data is in the three MSBs and the other five bits
; should be 0.  If any of them are not 0, we've likey run into some trouble.
; If so, re-initialize the DS2450.
; -----------------------------------------------------------------------------
        lda     mndray          ; get first byte
        ora     mndray+1        ; merge second byte
        ora     mndray+2        ; merge third byte
        ora     mndray+3        ; merge fourth byte
        and     #$1f            ; look only at lower 5 bits
        beq     iqd4ok          ; if all 0's, no erros here, check rest.

        jsr     iniaag          ; if not, re-initialize the DS2450
        bset    7,errdir        ; set the error flag
        bra     iqd4ok          ; and do the error processing

; -----------------------------------------------------------------------------
; Original Dallas Support
; -----------------------------------------------------------------------------

iqdr4d: lda     #$7f            ; command to turn 2407 channel B off
        bsr     iswich          ; go do it (no recourse if failure)
        beq     iqd4ok          ; if OK, no error here
        bset    7,errdir        ; otherwise set error flag

iqd4ok: brclr   7,errdir,iqd4ex ; if no cumulative errors, exit quietly
        inc     errdir          ; otherwise increment direction error counter
        bset    7,errdir        ; inc may clear error flag, set it again

iqd4ex: rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      dirstt
; Called By:    timrtn
; Calls:        None
; Gazintas:     windir (current wind direction (0-15))
; Gazoutas:     cumwdr (array of wind samples)
; Function:     This is the routine that accumulates all the wind samples over
;               a 5-minute period.  The averaging method that we use (defined
;               by Russ) needs to know how many samples occurred at each of
;               16 compass points and then does some magic.  We clear out this
;               array at the start and then increment the byte indexed by the
;               sampled wind speed.  That way we have a histogram of all the
;               samples for all the possible directions.
; -----------------------------------------------------------------------------
dirstt: brset   7,errdir,dirstx ; If error, don't tally statistics
        lda     #cumwdr         ; wind direction bins base
        add     windir          ; add offset
        tax                     ; make an index
        clrh                    ; make sure H is 0
        inc     0,x             ; go increment the bin
dirstx: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iswich
; Called By:    iqdir1, iqdir4
; Calls:        sndadr (send 1-wire address)
;               crcini (initialize 16-bit CRC)
;               i_writ (send 1-wire byte with CRC)
;               i_read (read 1-wire byte with CRC)
;               crcchk (check 16-bit CRC results)
; Gazintas:     A: byte to send with switch settings
; Gazoutas:     Z: set if success, cleared if failure
; Function:     Whenever an ibutton device gets connected to the bus after
;               being disconnected for a while, it sends out a reset pulse.
;               Under normal circumstances this is good, but for the direction
;               sensors that get connected or not based on the wind vane, this
;               would be a disaster, as every time the vane moved, the device
;               that gets connected would trash any communications that would
;               be going on.  The solution to the problem is to isolate the
;               direction ibuttons from the bus and connect them only when we
;               want to read the direction.  The DS2407 performs this function
;               and this is the routine to turn that swtich on (connected) and
;               off (disconnected).  CRC checking for this routine is mandatory
;               as we need absolute confirmation that we have closed and opened
;               the switch.  This is the routine that twiddles the switch to
;               read the direction chips.  The value passed in A is the value
;               to write to the switch register.  0x3f turns switch B on, 0x7f
;               turns it off.  The sequence is Address,55,07,00,sw,cc,cc.
; -----------------------------------------------------------------------------
iswich: sta     temp2           ; save the command
        lda     #8              ; up to 8 tries
        sta     iswcnt          ; save the count

iswlup: ldhx    #fisw           ; get the switch chip address
        jsr     sndadr          ; send it
        jsr     crcini          ; clear out the CRC
        lda     #$55            ; Write status command
        jsr     i_writ          ; send it
        lda     #$07            ; Status reg address LSB
        jsr     i_writ          ; send it
        lda     #$00            ; Status reg address MSB
        jsr     i_writ          ; send it
        lda     temp2           ; get our command back
        jsr     i_writ          ; send it off
        jsr     i_read          ; Get CRC LSB
        jsr     i_read          ; get CRC MSB

        jsr     crcchk          ; go check CRC result
        beq     iswex           ; if OK, exit
        dec     iswcnt          ; otherwise decrement attempt count
        bne     iswlup          ; loop for 15 tries before yelling uncle

        lda     #1              ; clear Z bit
iswex:  rts                     ; done (for now)

        page
; -----------------------------------------------------------------------------
; Routine:      match
; Called By:    iqdir2, iqdir3
; Calls:        None
; Gazintas:     mndray (address read from 1-wire bus)
;               fidir (array of eight sensor addresses)
; Gazoutas:     A: index (0-7) if match
;               C: 0=success, 1=no match
; Function:     This routine searches our list known direction IDs in flash
;               to find the match for the direction ID we read from the bus.
;               This routine tries to do a match and returns the number 0-7
;               depending on which entry in the list matches.  The idea is
;               that 0 is 0 degrees, 1 is 45 degrees, etc.  The list of
;               irections starts at address fidir1.
; -----------------------------------------------------------------------------
match:  clr     temp            ; our entry counter

matlup: lda     temp            ; get the entry counter
        asla                    ; times 2
        asla                    ; times 4
        asla                    ; times 8
        tax                     ; X is now offset from base
        clrh                    ; make sure H is 0

        lda     mndray          ; get first byte to compare
        cmp     fidir1+0,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop
        lda     mndray+1        ; get second byte to compare
        cmp     fidir1+1,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop
        lda     mndray+2        ; get third byte to compare
        cmp     fidir1+2,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop
        lda     mndray+3        ; get fourth byte to compare
        cmp     fidir1+3,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop
        lda     mndray+4        ; get fifth byte to compare
        cmp     fidir1+4,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop
        lda     mndray+5        ; get sixth byte to compare
        cmp     fidir1+5,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop
        lda     mndray+6        ; get seventh byte to compare
        cmp     fidir1+6,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop
        lda     mndray+7        ; get seventh byte to compare
        cmp     fidir1+7,x      ; compare to value from table
        bne     matlex          ; if not the same, exit loop

        lda     temp            ; we got a match, load the entry #
        clc                     ; carry bit clear to say success
        rts                     ; all done

matlex: inc     temp            ; no match, try next entry in table
        lda     temp            ; save it.
        cmp     #8              ; see if last entry
        bls     matlup          ; if still 0-7 range, try again
        sec                     ; carry bit set to say failure
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      amatch
; Called By:    ssens
; Calls:        None
; Gazintas:     mndray (address read from 1-wire bus)
;               msg++ (array of eight sensor addresses)
; Gazoutas:     A: index (0-7) if match
;               C: 0=success, 1=no match
; Function:     This routine is the same as match, except that the array of
;               addresses is in RAM (msg) instead of flash.  Note that this
;               routine uses more than just the msg buffer, it assumes the
;               entire 64-bytes starting with msg contains valid IDs.  See
;               the beginning of ram.asm for all the warnings and explanation.
; -----------------------------------------------------------------------------
amatch: clr     temp            ; our entry counter

amatlp: lda     temp            ; get the entry counter
        asla                    ; times 2
        asla                    ; times 4
        asla                    ; times 8
        tax                     ; X is now offset from base
        clrh                    ; make sure H is 0

        lda     mndray          ; get first byte to compare
        cmp     msg+0,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop
        lda     mndray+1        ; get second byte to compare
        cmp     msg+1,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop
        lda     mndray+2        ; get third byte to compare
        cmp     msg+2,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop
        lda     mndray+3        ; get fourth byte to compare
        cmp     msg+3,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop
        lda     mndray+4        ; get fifth byte to compare
        cmp     msg+4,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop
        lda     mndray+5        ; get sixth byte to compare
        cmp     msg+5,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop
        lda     mndray+6        ; get seventh byte to compare
        cmp     msg+6,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop
        lda     mndray+7        ; get seventh byte to compare
        cmp     msg+7,x         ; compare to value from table
        bne     amatlx          ; if not the same, exit loop

        lda     temp            ; we got a match, load the entry #
        clc                     ; carry bit clear to say success
        rts                     ; all done

amatlx: inc     temp            ; no match, try next entry in table
        lda     temp            ; save it.
        cmp     #8              ; see if last entry
        bls     amatlp          ; if still 0-7 range, try again
        sec                     ; carry bit set to say failure
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      piwdir
; Called By:    tncmsg, cmpmsg
; Calls:        none
; Gazintas:     windir (current wind direction (instant, 0-15))
;               A: Mode (0=numbers w/o 0's, 1=text, 2=numbers with 0's)
; Gazoutas:     atemp (wind direction in ASCII)
; Function:     This is the routine that converts the wind direction number to
;               something human readable.  This is only used for the direct
;               readings from the sensors, so this is a simple tranlsation of
;               the wind direction number (0-15) translated to text for the
;               eight points of the compass.  A passes the desired display
;               mode, either the compass points or numbers with or without
;               leading zeros.
; -----------------------------------------------------------------------------
piwdir: ldx     #48             ; 48 bytes per direction table
        mul                     ; x:a is offset
        add     #wdir0%256      ; add base LSB
        sta     cnv2a+1         ; save it for a bit
        txa                     ; get MSB
        adc     #wdir0\256      ; add MSBs with carry
        sta     cnv2a           ; save it

        lda     windir          ; get the direction number
        ldx     #3              ; multiplier on directions
        mul                     ; 3 bytes per direction (resut in a)
        add     cnv2a+1         ; add direction offset LSB
        tax                     ; save in X
        lda     cnv2a           ; get MSB
        adc     #0              ; add carry (if any)
        psha                    ; save it on stack
        pulh                    ; pop onto H.  H:X is direction!

        mov     x+,atemp+0      ; move first byte
        mov     x+,atemp+1      ; move second byte
        mov     x+,atemp+2      ; move third byte
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      pmwdir
; Called By:    showit
; Calls:        none
; Gazintas:     windir (current wind direction (instant, 0-15))
;               A: Mode (0=numbers w/o 0's, 1=text, 2=numbers with 0's)
; Gazoutas:     matemp (wind direction in ASCII)
; Function:     Mainline version of piwdir.  Same code, different variables
; -----------------------------------------------------------------------------
pmwdir: ldx     #48             ; 48 bytes per direction table
        mul                     ; x:a is offset
        add     #wdir0%256      ; add base LSB
        sta     macnv+1         ; save it for a bit
        txa                     ; get MSB
        adc     #wdir0\256      ; add MSBs with carry
        sta     macnv           ; save it

        lda     windir          ; get the direction number
        ldx     #3              ; multiplier on directions
        mul                     ; 3 bytes per direction (resut in a)
        add     macnv+1         ; add direction offset LSB
        tax                     ; save in X
        lda     macnv           ; get MSB
        adc     #0              ; add carry (if any)
        psha                    ; save it on stack
        pulh                    ; pop onto H.  H:X is direction!

        mov     x+,matemp+0     ; move first byte
        mov     x+,matemp+1     ; move second byte
        mov     x+,matemp+2     ; move third byte
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      avgdir
; Called By:    tncmsg
; Calls:        None
; Gazintas:     cumwdr (histogram of wind direction samples)
; Gazoutas:     atemp (wind direction in ASCII)
; Function:     This is the mambo routine that converts the histogram of
;               wind direction samples into an average direction.  Russ knows
;               why this needs doing this way and it is pretty cool  Follow
;               the comments as this code goes along and hopefuly all will
;               make sense.
; -----------------------------------------------------------------------------
;
;       This is the wind direction averaging routine.  I am doing what Russ
;       says to do, he is the expert.  Here is the theory (condensed).  I have
;       16 bins for each of the 16 possible directions the wind sensors can
;       detect.  For each 5-second sample, I increment that bin value by 1.
;       after 10 minutes (120 samples), I have a histogram of the samples in
;       each wind direction bin, lets call them bin(x) where 0<=x<=15.
;       Next I calculate the totals 16 groups of five bins to find out which
;       group of five has the most total samples.  I do some overflow stuff
;       so that bin(16:19) are the same as bin(0:4).  Amyway, mathematically,
;       we find the max value for 0<=x<=15 of the sum bin(x:x+4).  We'll call
;       the value of x which yields the max m.  There may be multiple entries
;       where this is the case, I'll just pick the first.
;       For M we take the weighted sum of the entries times the number of
;       samples in that entry to figure out the average direction of the
;       samples.  That is, we compute the sum i;bin(M+i) for 0<=1<=4.  We then
;       divide that value by the total samples in that range to ge the average.
;       So much for high level math stuff, on to real code to implement the
;       above theory.

;       First get the index that has the max total samples from it's base to
;       the base+4.  We do this by running two loops, the outer loop is from
;       15-0 for each possible starting value, then the inner loop is from
;       4-0 to accumulate the totals.  Since there is 120 samples max, we need
;       only a single byte for totals and use divdnd for that.  We also use
;       divdnd+1 to remember the index of the maximum series total.

avgdir: clr     divsor          ; max cumulative value
        clr     avgwdr          ; Average wind direction init value
        lda     #15             ; outer loop counter @ max
        sta     cntr            ; 0-15 outer loop counter
avdlp1: lda     #4              ; inner loop counter @ max
        sta     temp2           ; 0-4 inner loop counter
        clr     temp            ; Place for totals
avdlp2: lda     cntr            ; get outer loop counter
        add     temp2           ; add inner loop counter
        and     #$0f            ; subrange to 0-15
        tax                     ; make an index
        clrh                    ; make sure H is 0
        lda     temp            ; get accumulation so far
        add     cumwdr,x        ; add in next value
        sta     temp            ; save result
        dec     temp2           ; decrement
        bpl     avdlp2          ; complete inner loop

        lda     divsor          ; get current max total
        cmp     temp            ; compare to just-totaled max
        bhi     avdlpx          ; if current max is higher, on to next

        lda     temp            ; get new max
        sta     divsor          ; store it
        lda     cntr            ; get max start index
        sta     avgwdr          ; store it

avdlpx: dec     cntr            ; decrement outer loop
        bpl     avdlp1          ; do all 15

        lda     divsor          ; get the max count
        bne     avdn00          ; if not 0 (normal), go process

        jsr     inidot          ; all dots for invalid measurement
        bra     avdex           ; all done

;       Done finding the index with the max series, it is in avgwdr.
;       divsor is the total max value.
;       Now compute the weighted sum of this series.  This requires
;       16-bit math and I'll use the quotnt locations for the sum.  Note
;       that we don't bother with the first entry as it is multiplied
;       by 0.
;       MAX integer value 60 samples x 4 = 240

avdn00: clr     divdnd          ; clear total MSB
        clr     divdnd+1        ; clear total LSB
        lda     #4              ; loop counter
        sta     cntr            ; remember it

avdlp3: lda     cntr            ; get loop counter
        add     avgwdr          ; add offset to base
        and     #$0f            ; limit index to 0-15 (do wrap)
        tax                     ; make an index
        clrh                    ; make sure H is 0
        lda     cumwdr,x        ; get it's sample count
        ldx     cntr            ; get offset-only back
        mul                     ; x:a is samples weighted sum
        add     divdnd+1        ; add LSB to total
        sta     divdnd+1        ; save result
        txa                     ; get weighted sample MSB in a
        adc     divdnd          ; add to total
        sta     divdnd          ; save result
        dec     cntr            ; decrement loop counter
        bne     avdlp3          ; do 4-1 entries (not 0)

;       divdnd:divdnd+1 how has the weighted total of the series.
;       We now need to divide this by the total samples to get the
;       averaged fractional part of the wind direction. This is done
;       by integer division of the weighted samples (divdnd) by the
;       total samples in the series (divsor).  By sheer coincidence,
;       the variable are already in the right place!  To get better
;       accuracy with the end result, I need to do some creative math here
;       first.  I take our cumulative result and multiply it by 45 first.
;       That way, after the integer math I will get an answer that is
;       in half-degrees already.  Multiplying a 16-bit value is fun
;       in it's own right, but overflow can't happen so I'm safe.

        lda     divdnd+1        ; get weighted total LSB
        ldx     #45             ; the value to multiply by
        mul                     ; do it
        sta     divdnd+1        ; store multiplied LSB back
        lda     divdnd          ; get old MSB
        stx     divdnd          ; store multiplied MSB back
        ldx     #45             ; multiplier again
        mul                     ; multiply origitnal MSB by 45
        add     divdnd          ; add cumulative MSB to multipled MSB-LSB
        sta     divdnd          ; store result (MSB-MSB should be 0)

        jsr     divu            ; quotnt,remndr=divdnd/divsor

;       After all that fun, quotnt should be the offset in half-degrees
;       from the base index degree value.  All that is left is to convert
;       the base index to half degrees by multiplying it by 45, adding
;       the offset just computed, and dividing by two.

        lda     avgwdr          ; get the base vale
        and     #$0f            ; make sure it is 0-15
        ldx     #45             ; converter to real half-degrees
        mul                     ; make it so
        add     quotnt+1        ; add offset LSB
        sta     divdnd+1        ; save result for conversion later
        txa                     ; get base MSB in A reg
        adc     quotnt          ; add offset MSB (and carry)
        sta     divdnd          ; save result for conversion later

        lsr     divdnd          ; divide result MSB by two
        ror     divdnd+1        ; divide LSB by two too

;       We now have the direction the wind is blowing, except it may be more
;       than 360 degrees.  See if it is, and adjust as necessary.  360 is
;       0x0168, so see if we are greater or equal to that and if so, subtract
;       it.

        lda     divdnd          ; get the MSB
        beq     avddis          ; if 0, no chance of overflow
        cmp     #$01            ; compare MSB
        bhi     avd360          ; if >1 (I think impossible) subtract

        lda     divdnd+1        ; if MSB 1, we need to look at the LSB
        cmp     #$68            ; the limit
        blo     avddis          ; if less than 0x168 (360), go convert

avd360: lda     divdnd+1        ; get the direction
        sub     #$68            ; subtract 360 LSB
        sta     divdnd+1        ; save result
        lda     divdnd          ; get the direction MSB
        sbc     #1              ; subtract 360 MSB (and carry)
        sta     divdnd          ; save result--done

avddis: jsr     bin2as          ; go convert direction to ASCII

avdex:  rts                     ; done, atemp is ASCII value

        page
; -----------------------------------------------------------------------------
; Routine:      rddir
; Called By:    ssens
; Calls:        iqdir1,2,3,4
; Gazintas:     None
; Gazoutas:     windir
; Function:     This is a routine to read the wind direction outside the
;               intrrupt routine. This is used by the setup routine to read
;               a position while doing the direction calibration.  All I do
;               is call each of the four interrupt routines in sequence,
;               understanding that they are not being interrupt driven.  The
;               only tricky bit is that since the normal routines are
;               expecting to be called as part of an interrupt, they do not
;               assume they can be interrupted.  For the bit timing there are
;               some sensitive timings that cannot be interrupted.  My solution
;               is to mostly disable interrupts, but then re-enable them for a
;               bit between routine calls to support any timer interrupts that
;               may come along.  Maybe not the most elegant, but it should work
;               and is better than disabling interrupts entirely.
; -----------------------------------------------------------------------------


rddir:  sei                     ; disable interrupts
        jsr     iqdir1          ; open channel
        cli                     ; enable interrupts for a bit

        lda     #2              ; 2ms
        jsr     waitms          ; wait it (needed for AAG conversion)

        sei                     ; disable interrupts again
        jsr     iqdir2          ; read magnet #1
        cli                     ; enable interrupts for a bit

        nop                     ; the bit

        sei                     ; disable interrupts again
        jsr     iqdir3          ; read magnet #2
        cli                     ; enable interrupts for a bit

        nop                     ; the bit

        sei                     ; disable interrupts again
        jsr     iqdir4          ; close channel
        cli                     ; enable interrupts

        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iniaag
; Called By:    dowx
; Calls:
; Gazintas:     None
; Gazoutas:     None
; Function:     This routine initializes the AAG wind direction sensor if it
;               exists.  If not, it just exits.  For each of the four channels
;               the A/D is set up as follows:
;                 Convert 3 bits
;                 Don't drive (input mode)
;                 5V input range
;                 All alarms off
;                 Clear POR flag
;               This routine only tries once to initialize the DS2438.  If it
;               fails, it gives up.  I do this as this routine gets called
;               whenever I get a serious read error and can't hang the system.
; -----------------------------------------------------------------------------
iniaag: lda     fisw            ; get switch/ADC family ID
        cmp     #$20            ; see if AAG-based setup
        bne     iniaax          ; if not, exit

        lda     #$08            ; Address LSB
        sta     divdnd          ; remember it

        ldhx    #fisw           ; get the A/D chip address
        jsr     sndadr          ; send it
        jsr     crcini          ; clear out the CRC
        lda     #$55            ; Write memory command
        jsr     i_writ          ; send it
        lda     divdnd          ; Write address LSB
        jsr     i_writ          ; send it
        lda     #$00            ; Write address MSB (always 0)
        jsr     i_writ          ; send it
inaagl: lda     #$03            ; 0008=03=3 bits, don't drive output
        jsr     i_writ          ; send it

        jsr     i_read          ; get CRC LSB
        jsr     i_read          ; get CRC MSB
        jsr     crcchk          ; See if we get the correct answer
        bne     iniaer          ; if not, error

        jsr     i_read          ; read data back (already verified)

        inc     divdnd          ; increment address LSB
        clr     crc16h          ; re-initialize CRC to address (MSB)
        mov     divdnd,crc16l   ; re-initialize CRC to address (LSB)
        lda     #$01            ; 5V range, no alarms, clear POR
        jsr     i_writ          ; write it

        jsr     i_read          ; get CRC LSB
        jsr     i_read          ; get CRC MSB
        jsr     crcchk          ; See if we get the correct answer
        bne     iniaer          ; if not, error

        jsr     i_read          ; read data back (already verified)

        lda     divdnd          ; get the address
        cmp     #$0f            ; see if done (adress $0f is last)
        beq     iniaax          ; if $10, done

        inc     divdnd          ; increment address LSB
        clr     crc16h          ; re-initialize CRC to address (MSB)
        mov     divdnd,crc16l   ; re-initialize CRC to address (LSB)
        bra     inaagl          ; go do next channel.

iniaax: clc                     ; C is OK flag
        rts                     ; all done
iniaer: sec                     ; C* is not OK flag
        rts                     ; exit
